Package: gapclosing
Title: Estimate Gaps Under an Intervention
Version: 1.0.3
Authors@R: 
    person(given = "Ian",
           family = "Lundberg",
           role = c("aut", "cre"),
           email = "ianlundberg@ucla.edu",
           comment = c(ORCID = "https://orcid.org/0000-0002-1909-2270"))
Description: Provides functions to estimate the disparities across categories (e.g. Black and white) that persists if a treatment variable (e.g. college) is equalized. Makes estimates by treatment modeling, outcome modeling, and doubly-robust augmented inverse probability weighting estimation, with standard errors calculated by a nonparametric bootstrap. Cross-fitting is supported. Survey weights are supported for point estimation but not for standard error estimation; those applying this package with complex survey samples should consult the data distributor to select an appropriate approach for standard error construction, which may involve calling the functions repeatedly for many sets of replicate weights provided by the data distributor. The methods in this package are described in the accompanying paper: <doi:10.1177/00491241211055769>.
License: MIT + file LICENSE
URL: https://ilundberg.github.io/gapclosing/
BugReports: https://github.com/ilundberg/gapclosing/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: stats, mgcv, ranger, glmnet, magrittr, dplyr, forcats,
        ggplot2, tidyr, foreach, tidyselect, parallel, doParallel,
        doRNG
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, spelling
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-22 23:36:28 UTC; ilundberg
Author: Ian Lundberg [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1909-2270>)
Maintainer: Ian Lundberg <ianlundberg@ucla.edu>
Config/testthat/edition: 3
Language: en-US
Repository: CRAN
Date/Publication: 2026-05-23 11:00:20 UTC
Built: R 4.5.2; ; 2026-05-25 02:18:07 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,gapclosing)
S3method(plot,gapclosing)
S3method(print,gapclosing)
S3method(summary,gapclosing)
export("%>%")
export(cross_fit_estimator)
export(df_to_gapclosing_list)
export(disparityplot)
export(fit_ridge)
export(gapclosing)
export(generate_simulated_data)
export(pairwise_diff)
export(point_estimator)
importFrom(doRNG,"%dorng%")
importFrom(foreach,"%do%")
importFrom(magrittr,"%>%")

# gapclosing

<!-- badges: start -->
  [![CRAN status](https://www.r-pkg.org/badges/version/gapclosing)](https://CRAN.R-project.org/package=gapclosing) [![](https://cranlogs.r-pkg.org/badges/gapclosing)](https://cran.r-project.org/package=gapclosing)
  <!-- badges: end -->

Estimate Gaps Under an Intervention

# Summary

Provides functions to estimate gap-closing estimands: the disparities across categories (e.g. Black and white) that persists if a treatment variable (e.g. college) is equalized. The purpose is to estimate the average outcomes that units would realize if exposed to a counterfactual treatment assignment rule.

The package will enable the user to:

* Estimate treatment and outcome prediction functions with Generalized Linear Models, Generalized Additive Models, ridge regression, or random forest
* Combine those in doubly-robust estimators of gap-closing estimands
* Produce confidence intervals by the bootstrap
* Visualize the result in plots

# Installation instructions

Install from CRAN with one line: `install.packages("gapclosing")`.

To install the latest development version,

1. First, install the `devtools` package: `if(!require(devtools)) install.packages("devtools")`
2. Then, install the `gapclosing` package with the command `devtools::install_github("ilundberg/gapclosing")`.

# Getting started

To get started, see the [vignette](https://ilundberg.github.io/gapclosing/doc/gapclosing.html). Also see the [working paper](https://doi.org/10.31235/osf.io/gx4y3) for which this package is the software implementation.

>Lundberg, Ian. Forthcoming. "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." _Sociological Methods and Research_. Draft available at [https://doi.org/10.31235/osf.io/gx4y3](https://doi.org/10.31235/osf.io/gx4y3).

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(dpi = 300, fig.width = 6.5, fig.height = 3, out.width = "650px")

## ----include = FALSE----------------------------------------------------------
t0 <- Sys.time()

## ----message = FALSE, warning = FALSE-----------------------------------------
set.seed(08544)
library(gapclosing)
library(dplyr)
library(ggplot2)

## -----------------------------------------------------------------------------
simulated_data <- generate_simulated_data(n = 1000)
head(simulated_data)

## -----------------------------------------------------------------------------
estimate <- gapclosing(
  data = simulated_data,
  counterfactual_assignments = 1,
  outcome_formula = formula(outcome ~ confounder + category*treatment),
  treatment_formula = formula(treatment ~ confounder + category),
  category_name = "category",
  se = TRUE,
  # You can process the bootstrap in parallel with as many cores as available
  parallel_cores = 2
)

## ----include = FALSE----------------------------------------------------------
plots <- plot(estimate, return_plots = TRUE)

## ----echo = FALSE, fig.width = 5, fig.height = 3.5, out.width = "650px", fig.cap = "Figure 1 produced by plot() function"----
print(plots[[1]] +
        ggtitle("First result of a call to plot()"))

## ----echo = FALSE, fig.width = 5, fig.height = 3.5, out.width = "650px", fig.cap = "Figure 1 produced by plot() function"----
disparityplot(estimate, category_A = "A", category_B = "B") +
  ggtitle("A disparityplot()")

## ----include = FALSE--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
old <- options()
options(width = 300)

## ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
summary(estimate)

## ----include = FALSE----------------------------------------------------------
options(old)

## ----echo = FALSE, fig.height = 2---------------------------------------------
for_DAG <- data.frame(label = c("Category","Confounder","Treatment","Outcome","Unobserved"),
                      x = c(1,2,3,4,1),
                      y = c(1,1,1,1,0))
for_DAG %>%
  ggplot(aes(x = x, y = y, label = label)) +
  # Edges coming out of category
  annotate(geom = "segment",
         x = 1, y = 1, xend = 1.55, yend = 1,
         arrow = arrow(length = unit(.1,"in"))) +
  annotate(geom = "curve",
           x = 1, y = 1, xend = c(2.9,3.9), yend = c(1.2,1.2),
           arrow = arrow(length = unit(.1,"in")),
           curvature = -.3) +
  # Edges coming out of confounder
  annotate(geom = "segment",
           x = 2, y = 1, xend = 2.65, yend = 1,
           arrow = arrow(length = unit(.1,"in"))) +
  annotate(geom = "curve",
           x = 2, y = 1, xend = 3.75, yend = .82,
           arrow = arrow(length = unit(.1,"in")),
           curvature = .25) +
  # Edges coming out of treatment
  annotate(geom = "segment",
           x = 3, y = 1, xend = 3.65, yend = 1,
           color = "blue", size = 1.05,
           arrow = arrow(length = unit(.13,"in"))) +
  # Edges coming out of unobserved
  annotate(geom = "segment",
           x = 1, y = 0, xend = c(1,1.9), yend = .8,
           arrow = arrow(length = unit(.1,"in"))) +
  annotate(geom = "curve",
           x = 1, y = 0, xend = c(3.9), yend = .8,
           arrow = arrow(length = unit(.1,"in")),
           curvature = .25) +
  geom_label(color = "white") +
  geom_label(data = for_DAG %>% filter(label %in% c("Category","Confounder"))) +
  geom_text(data = for_DAG %>% filter(label %in% c("Unobserved","Treatment","Outcome"))) +
  ylim(c(-.3,2)) +
  xlim(c(0,5)) +
  theme_void()

## ----warning = FALSE----------------------------------------------------------
estimate_gam <- gapclosing(
  data = simulated_data,
  counterfactual_assignments = 1,
  outcome_formula = formula(outcome ~ s(confounder) + category*treatment),
  treatment_formula = formula(treatment ~ s(confounder) + category),
  category_name = "category",
  treatment_algorithm = "gam",
  outcome_algorithm = "gam",
  sample_split = "cross_fit"
  # Note: Standard errors with `se = TRUE` are supported.
  # They are omitted here only to speed vignette build time.
)

## -----------------------------------------------------------------------------
estimate_ranger <- gapclosing(
  data = simulated_data,
  counterfactual_assignments = 1,
  outcome_formula = formula(outcome ~ confounder + category),
  treatment_formula = formula(treatment ~ confounder + category),
  category_name = "category",
  treatment_algorithm = "ranger",
  outcome_algorithm = "ranger",
  sample_split = "cross_fit"
  # Note: Standard errors with `se = TRUE` are supported.
  # They are omitted here only to speed vignette build time.
)

## ----include = FALSE----------------------------------------------------------
glm_plot <- plot(estimate, return_plots = T)[[1]]
gam_plot <- plot(estimate_gam, return_plots = T)[[1]]
ranger_plot <- plot(estimate_ranger, return_plots = T)[[1]]

## ----echo = FALSE, figures-side, fig.show="hold", out.width="30%", message = FALSE, warning = FALSE----
glm_plot +
    ggtitle("GLM estimate") +
    ylim(c(-1.5,1.5)) +
  theme(legend.position = "bottom")
gam_plot +
    ggtitle("GAM estimate") +
    ylim(c(-1.5,1.5)) +
  theme(legend.position = "bottom")
ranger_plot +
  ggtitle("Random forest estimate") +
  ylim(c(-1.5,1.5)) +
  theme(legend.position = "bottom")

## -----------------------------------------------------------------------------
estimate_stochastic <- gapclosing(
  data = simulated_data,
  counterfactual_assignments = .75,
  outcome_formula = formula(outcome ~ confounder + category*treatment),
  treatment_formula = formula(treatment ~ confounder + category),
  category_name = "category"
)

## ----echo = FALSE, fig.caption = "A conceptual figure for which the package does not provide a general-purpose function."----
counterfactual_assignments_values <- seq(0,1,.25)
many_stochastic_estimates <- rep(NA, length(counterfactual_assignments_values))
for (i in 1:length(counterfactual_assignments_values)) {
  estimate_case <- gapclosing(
    data = simulated_data,
    counterfactual_assignments = counterfactual_assignments_values[i],
    outcome_formula = formula(outcome ~ confounder + category*treatment),
    treatment_formula = formula(treatment ~ confounder + category),
    category_name = "category"
  )
  estimate_we_want <- estimate_case$change_disparities %>%
    filter(change_type == "proportional") %>%
    filter(category == "B - A")
  many_stochastic_estimates[i] <- estimate_we_want$estimate
}
data.frame(counterfactual_assignments = counterfactual_assignments_values,
           estimate = many_stochastic_estimates) %>%
  ggplot(aes(x = counterfactual_assignments,
             y = estimate)) +
  geom_hline(yintercept = c(0,1),
             color = "gray", linetype = "dashed") +
  geom_line(color = "gray") +
  geom_point(aes(color = factor(counterfactual_assignments == .75))) +
  xlab("Counterfactual Treatment Assignment Probability") +
  scale_y_continuous(name = "Percent of B - A Gap Closed",
                     labels = function(x) paste0(round(100*x),"%")) +
  annotate(geom = "text",
           x = 1, y = c(0,1),
           label = c("No change to disparity",
                     "Disparity completely eliminated"),
           color = "darkgray", size = 3, hjust = 1, vjust = 1.5) +
  annotate(geom = "text",
           x = .1, y = .3, 
           label = "Disparities eliminated under various\nstochastic\ninterventions",
           color = "gray", size = 3, hjust = 0, vjust = 1) +
  annotate(geom = "text",
           x = .75, y = .3,
           label = "Gap-Closing Estimand\nCalculated Above",
           color = "blue", size = 3, vjust = -.5) +
  scale_color_manual(values = c("gray","blue")) +
  theme_bw() +
  theme(panel.grid = element_blank(),
        legend.position = "none")

## -----------------------------------------------------------------------------
our_assignments <- case_when(simulated_data$category == "A" ~ .5,
                             simulated_data$category == "B" ~ .4, 
                             simulated_data$category == "C" ~ .3)
estimate_stochastic <- gapclosing(
  data = simulated_data,
  counterfactual_assignments = our_assignments,
  outcome_formula = formula(outcome ~ confounder + category*treatment),
  treatment_formula = formula(treatment ~ confounder + category),
  category_name = "category"
)

#' Generate simulated data
#' @description Generates simulated data to illustrate the gapclosing function
#' @param n Number of observations to be generated
#' @return A data frame with \code{n} rows and 4 columns containing simulated data containing \code{category} over which disparities are defined, a \code{confounder} that affects treatment assignment, a binary \code{treatment}, and a continuous \code{outcome}.
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @export

generate_simulated_data <- function(n = 1000) {
  # Initialize non-standard evaluation variables to avoid R CMD check warnings.
  confounder <- treatment <- treatment_effect <- outcome <- NULL
  data.frame(category = sample(c("A","B","C"), n, replace = TRUE)) %>%
    # Create a confounding variable called confounder
    dplyr::mutate(confounder = stats::rnorm(n,
                                            mean = dplyr::case_when(category == "A" ~ -1.5,
                                                                    category == "B" ~ 0,
                                                                    category == "C" ~ 1.5),
                                            sd = 1),
                  # Assign treatment as a function of the confounder
                  treatment = stats::rbinom(n, size = 1, prob = stats::plogis(confounder)),
                  # Create a treatment effect that varies by category
                  treatment_effect = dplyr::case_when(category == "A" ~ 1.3,
                                                      category == "B" ~ 0,
                                                      category == "C" ~ -1.2),
                  # Generate the outcome
                  outcome = stats::rnorm(n, mean = confounder + treatment*treatment_effect, sd = 1)) %>%
    # Remove the treatment effect, which one would not know in reality
    dplyr::select(-treatment_effect)
}

test_that("list to df to list inverts correctly", {
  simulated_data <- generate_simulated_data(n = 100)
  estimate <- gapclosing(
    data = simulated_data,
    outcome_formula = formula(outcome ~ treatment * category + confounder),
    treatment_name = "treatment",
    category_name = "category",
    counterfactual_assignments = 1,
    se = FALSE
  )
  estimate.df <- as.data.frame(estimate)
  estimate.list <- df_to_gapclosing_list(estimate.df)

  # Check that classes are correct
  expect_s3_class(estimate, "gapclosing")
  expect_s3_class(estimate.df, "data.frame")
  expect_equal(class(estimate.list), "list")

  # Check that components are equal
  # Because names in some cases get re-ordered, write a function to do this.
  check_component <- function(original, reconstructed) {
    check1 <- all(sort(names(original)) == sort(names(reconstructed)))
    expect_equal(check1, TRUE)
    expect_equal(original, reconstructed[names(original)])
  }
  for (i in 1:6) {
    check_component(estimate[[i]], estimate.list[[i]])
  }
  for (i in 1:3) {
    check2 <- all(sort(names(estimate$all_estimators[[i]])) == sort(names(estimate.list$all_estimators[[i]])))
    expect_equal(check2, TRUE)
    for (j in names(estimate$all_estimators[[i]])) {
      check_component(estimate$all_estimators[[i]][[j]], estimate.list$all_estimators[[i]][[j]])
    }
  }
})

#' Gap closing estimator
#' @description A function to estimate gap-closing estimands: means and disparities across categories of units that would persist under some counterfactual assignment of a treatment. To use this function, the user provides a data frame \code{data}, a rule \code{counterfactual_assignments} for counterfactually assigning treatment, a treatment and/or an outcome model for learning statistically about the counterfactuals, and the \code{category_name} of the variable in \code{data} over which categories are defined. The returned object summarizes factual and counterfactual means and disparities. Supported estimation algorithms include generalized linear models, ridge regression, generalized additive models, and random forests. Standard errors are supported by bootstrapping.
#' @param data Data frame containing the observed data
#' @param counterfactual_assignments Numeric scalar or vector of length nrow(data), each element of which is on the \[0,1\] interval. If a scalar, the counterfactual probability by which all units are assigned to treatment condition 1. If a vector, each element i corresponds to the counterfactual probability by which each unit i is assigned to treatment condition 1.
#' @param outcome_formula Outcome formula , in the style \code{outcome ~ treatment*covariate}. Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph). If \code{outcome_algorithm} = "ranger", then the outcome model will be fit separately on the treatment and control groups. Otherwise, the user must specify all interactions in the formula.
#' @param treatment_formula Treatment formula, in the style \code{treatment ~ covariate}. Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph).
#' @param category_name Character name of the variable indicating the categories over which the gap is defined. Must be the name of a column in \code{data}.
#' @param outcome_name Character name of the outcome variable. Only required when there is no outcome_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.
#' @param treatment_name Character name of the treatment variable. Only required when there is no treatment_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.
#' @param treatment_algorithm Character name of the algorithm for the treatment model. One of "glm", "ridge", "gam", or "ranger". Defaults to "glm", which is a logit model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}). If "ranger", this function avoids propensity scores equal to 0 or 1 by bottom- and top-coding predicted values at .001 and .999.
#' @param outcome_algorithm Character name of the algorithm for the outcome model. One of "lm", "ridge", "gam", or "ranger". Defaults to "lm", which is an OLS model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}).
#' @param sample_split Character for the type of sample splitting to be conducted. One of "single_sample" or "cross_fit". Defaults to "single_sample", in which case \code{data} is used for both learning the nuisance functions and aggregating to an estimate. Option "cross_fit" uses cross-fitting to repeatedly use part of the sample to learn the nuisance function and another part to estimate the estimand, averaged over repetitions with these roles swapped.
#' @param se Logical indicating whether standard errors should be calculated. Default is FALSE. Standard errors assume a simple random sample by default; to stratify by (category x treatment), see the \code{bootstrap_method} argument. Because many datasets are not simple random samples, users should carefully consider whether a simple random sample bootstrap will accurately capture uncertainty.
#' @param bootstrap_samples Only used if \code{se = TRUE}. Number of bootstrap samples. Default is 1000.
#' @param bootstrap_method Only used if \code{se = TRUE}. A character string stating how to conduct bootstrap samples. If "simple", then samples are drawn with replacement from the full data. If "stratified", then the bootstrap is carried out within subpopulations defined by category and treatment. The latter may be useful if the sample contains only a small number of observations in these cells and the user wants to ensure that every (category x treatment) cell appears in every bootstrap sample. With "stratified", inference assumes that in repeated samples from the true population the proportion in each (category x treatment) cell would not change; this may or may not correspond to the true sampling process. Users should be cautious.
#' @param parallel_cores Integer number of cores for parallel processing of the bootstrap. Defaults to sequential processing.
#' @param weight_name Character name of a sampling weight variable, if any, which captures the inverse probability of inclusion in the sample. The default assumes a simple random sample (all weights equal).
#' @param n_folds Only used if \code{method} = "cross_fit" and if \code{folds} is not provided. Integer scalar containing number of cross-validation folds. The function will assign observations to folds systematically: sort the data by the variable named \code{category_name}, then by the treatment variable, then at random. On this sorted dataset, folds are assigned systematically by repeated \code{1:n_folds}. To be used if the user does not provide \code{folds}. Defaults to 2.
#' @param folds_name Only used if \code{method} = "cross_fit". Character string indicating a column of \code{data} containing fold identifiers. This may be preferable to \code{n_folds} if the researcher has a reason to assign the folds in these data by some other process, perhaps due to particulars of how these data were generated. If null (the default), folds are assigned as stated in \code{n_folds}.
#' @return An object of S3 class \code{gapclosing}, which supports \code{summary()}, \code{print()}, and \code{plot()} functions. The returned object can be coerced to a data frame of estimates with \code{as.data.frame()}. \cr
#' The object returned by a call to \code{gapclosing} contains several elements.
#' \itemize{
#' \item \code{factual_means} A tibble containing the factual mean outcome in each category \cr
#' \item \code{factual_disparities} A tibble containing the disparities in factual mean outcomes across categories \cr
#' \item \code{counterfactual_means} A tibble containing the counterfactual mean outcome (post-intervention mean) in each category \cr
#' \item \code{counterfactual_disparities} A tibble containing the counterfactual disparities (gap-closing estimands) across categories \cr
#' \item \code{change_means} A tibble containing the additive and proportional change from factual to counterfactual values for mean outcomes \cr
#' \item \code{change_disparities} A tibble containing the additive and proportional change from factual to counterfactual values for disparities in mean outcomes (e.g. proportion of the factual gap which is closed by the intervention) \cr
#' \item \code{all_estimators} A list containing estimates by treatment modeling, outcome modeling, and doubly-robust estimation. If any of these are not applicable, estimates are NA. \cr
#' \item \code{primary_estimator_name} The name of the primary estimator (treatment_modeling, outcome_modeling, or doubly_robust). The estimates reported in the first 6 slots of the returned object come from this estimator. \cr
#' \item \code{treatment_model} The fitted treatment model (or models on each fold in the case of cross-fitting). Note that this model object is a point estimate with standard errors derived from the algorithm used to fit it; any standard errors within \code{treatment_model} do not come from bootstrapping by the package. \cr
#' \item \code{outcome_model} The fitted outcome model (or models on each fold in the case of cross-fitting). Note that this model object is a point estimate with standard errors derived from the algorithm used to fit it; any standard errors within \code{treatment_model} do not come from bootstrapping by the package. \cr
#' \item \code{call} The call that produced this \code{gapclosing} object \cr
#' \item \code{arguments} A list of all arguments from the call to \code{gapclosing}
#' }
#'
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @references Friedman J, Hastie T, Tibshirani R (2010). "Regularization Paths for Generalized Linear Models via Coordinate Descent." Journal of Statistical Software, 33(1), 1–22. {https://www.jstatsoft.org/htaccess.php?volume=33&type=i&issue=01}.
#' @references Wood S (2017). Generalized Additive Models: An Introduction with R, 2 edition. Chapman and Hall/CRC.
#' @references Wright MN, Ziegler A (2017). "ranger: A Fast Implementation of Random Forests for High Dimensional Data in C++ and R." Journal of Statistical Software, 77(1), 1–17. doi: 10.18637/jss.v077.i01.
#' @importFrom foreach %do%
#' @importFrom doRNG %dorng%
#' @importFrom magrittr %>%
#' @export
#'
#' @examples
#' # Simulate example data
#' simulated_data <- generate_simulated_data(n = 100)
#'
#' # Fit by outcome modeling
#' # You can add standard errors with se = TRUE
#' estimate <- gapclosing(
#'   data = simulated_data,
#'   outcome_formula = outcome ~ treatment * category + confounder,
#'   treatment_name = "treatment",
#'   category_name = "category",
#'   counterfactual_assignments = 1
#' )
#' summary(estimate)
#'
#' # Fit by treatment modeling
#' # You can add standard errors with se = TRUE
#' estimate <- gapclosing(
#'   data = simulated_data,
#'   treatment_formula = treatment ~ category + confounder,
#'   outcome_name = "outcome",
#'   category_name = "category",
#'   counterfactual_assignments = 1
#' )
#' summary(estimate)
#'
#' # Fit by doubly-robust estimation
#' # You can add standard errors with se = TRUE
#' estimate <- gapclosing(
#'   data = simulated_data,
#'   outcome_formula = outcome ~ treatment * category + confounder,
#'   treatment_formula = treatment ~ category + confounder,
#'   category_name = "category",
#'   counterfactual_assignments = 1
#' )
#' summary(estimate)
#'
#' # Fit by doubly-robust cross-fitting estimation with random forests
#' # You can add standard errors with se = TRUE
#' estimate <- gapclosing(
#'   data = simulated_data,
#'   outcome_formula = outcome ~ category + confounder,
#'   treatment_formula = treatment ~ category + confounder,
#'   category_name = "category",
#'   counterfactual_assignments = 1,
#'   outcome_algorithm = "ranger",
#'   treatment_algorithm = "ranger",
#'   sample_split = "cross_fit"
#' )
#' summary(estimate)

gapclosing <- function(
  data,
  counterfactual_assignments,
  outcome_formula = NULL,
  treatment_formula = NULL,
  category_name,
  outcome_name = NULL,
  treatment_name = NULL,
  treatment_algorithm = "glm",
  outcome_algorithm = "lm",
  sample_split = "single_sample",
  se = FALSE,
  bootstrap_samples = 1000,
  bootstrap_method = "simple",
  parallel_cores = NULL,
  weight_name = NULL,
  n_folds = 2,
  folds_name = NULL
) {

  ############################
  # Save arguments to return #
  ############################

  arguments <- list(
    data = data,
    counterfactual_assignments = counterfactual_assignments,
    outcome_formula = outcome_formula,
    treatment_formula = treatment_formula,
    category_name = category_name,
    outcome_name = outcome_name,
    treatment_name = treatment_name,
    treatment_algorithm = treatment_algorithm,
    outcome_algorithm = outcome_algorithm,
    sample_split = sample_split,
    se = se,
    bootstrap_samples = bootstrap_samples,
    bootstrap_method = bootstrap_method,
    parallel_cores = parallel_cores,
    weight_name = weight_name,
    n_folds = n_folds,
    folds_name = folds_name
  )

  # Initialize some objects for non-standard evaluation
  gapclosing.weight <- estimate <- f <- method <- additive <-
    proportional <- change_type <- change_formula <- method <- i <- NULL

  ############################
  # Check validity of inputs #
  ############################

  if (!(length(counterfactual_assignments) %in% c(1,nrow(data)))) {
    stop("ERROR: counterfactual_assignments must be length 1 implying the same assignment for everyone or nrow(data) giving the counterfactual assignment probability for each unit.")
  }
  if (!all(counterfactual_assignments >= 0 & counterfactual_assignments <= 1)) {
    stop("ERROR: counterfactual_assignments must be numeric values for probabilities in the range from 0 to 1")
  }
  if (is.null(treatment_formula) & is.null(outcome_formula)) {
    stop("ERROR: At least one of treatment_formula and outcome_formula must be non-null")
  }
  if (is.null(treatment_formula) & is.null(treatment_name)) {
    stop("ERROR: If no treatment_formula is given, you must specify treatment_name")
  }
  if (is.null(outcome_formula) & is.null(outcome_name)) {
    stop("ERROR: If no outcome_formula is given, you must specify outcome_name")
  }
  if (!is.character(category_name)) {
    stop("ERROR: category_name must be a character string")
  }
  if (!(category_name %in% colnames(data))) {
    stop("ERROR: category_name needs to be the name of a column in data")
  }
  if (!(treatment_algorithm %in% c("glm","ridge","gam","ranger"))) {
    stop("ERROR: treatment_algorithm must be one of glm, ridge, gam, ranger")
  }
  if (!(outcome_algorithm %in% c("lm","ridge","gam","ranger"))) {
    stop("ERROR: outcome_algorithm must be one of lm, ridge, gam, ranger")
  }
  if (!(sample_split %in% c("single_sample", "cross_fit"))) {
    stop("ERROR: sample_split must be one of single_sample, cross_fit")
  }
  if (!is.logical(se)) {
    stop("ERROR: se must be TRUE or FALSE")
  }
  if (!is.numeric(bootstrap_samples)) {
    stop("ERROR: bootstrap_samples should be numeric")
  }
  if (se & bootstrap_samples < 1000) {
    warning("You have asked for bootstrap standard errors (se = TRUE) but you have set bootstrap_samples to fewer than 1,000. Although the necessary number of bootstrap samples will differ across applications, you may need to consider increasing bootstrap_samples.")
  }
  if (!(bootstrap_method %in% c("simple","stratified"))) {
    stop("ERROR: bootstrap_method should be one of simple or stratified")
  }
  if (!is.null(weight_name)) {
    if (!is.character(weight_name)) {
      stop("ERROR: weight_name should be a character string")
    } else if (!(weight_name %in% colnames(data))) {
      stop("ERROR: weight_name should be a column in data")
    }
  }
  if (sample_split == "cross_fit" & is.na(n_folds) & is.null(folds_name)) {
    stop("ERROR: For cross fitting, one of n_folds or folds_name is required")
  } else if (sample_split == "cross_fit" & is.null(folds_name) & !is.numeric(n_folds)) {
    stop("ERROR: n_folds should be numeric")
  }
  if (!is.null(folds_name)) {
    if (!(folds_name %in% names(data))) {
      stop("ERROR: folds_name should be a column of data")
    }
  }

  #################
  # Modify inputs #
  #################

  # Get treatment and outcome names if they are not given
  if (is.null(treatment_name)) {
    treatment_name <- as.character(treatment_formula)[2]
    if (!(treatment_name %in% colnames(data))) {
      stop("ERROR: treatment_name was extracted from treatment_formula, but it appears to not be a column in data")
    }
  } else if (!(treatment_name %in% colnames(data))) {
    stop("ERROR: treatment_name was provided by the user, but it appears to not be a column in data")
  }
  if (is.null(outcome_name)) {
    outcome_name <- as.character(outcome_formula)[2]
    if (!(outcome_name %in% colnames(data))) {
      stop("ERROR: outcome_name was extracted from outcome_formula, but it appears to not be a column in data")
    }
  } else if (!(outcome_name %in% colnames(data))) {
    stop("ERROR: outcome_name was provided by the user, but it appears to not be a column in data")
  }
  # If the treatment is logical, make it numeric
  if (is.logical(data[[treatment_name]])) {
    data[[treatment_name]] <- as.numeric(data[[treatment_name]])
  }

  ##########################################
  # Make the counterfactual point estimate #
  ##########################################

  if (sample_split == "single_sample") {
    counterfactual_point <- point_estimator(
      data_learn = data,
      data_estimate = data,
      outcome_formula = outcome_formula,
      treatment_formula = treatment_formula,
      outcome_name = outcome_name,
      treatment_name = treatment_name,
      category_name = category_name,
      counterfactual_assignments = counterfactual_assignments,
      weight_name = weight_name,
      treatment_algorithm = treatment_algorithm,
      outcome_algorithm = outcome_algorithm
    )
  }
  if (sample_split == "cross_fit") {
    counterfactual_point <- cross_fit_estimator(
      data = data,
      counterfactual_assignments = counterfactual_assignments,
      outcome_formula = outcome_formula,
      treatment_formula = treatment_formula,
      category_name = category_name,
      outcome_name = outcome_name,
      treatment_name = treatment_name,
      treatment_algorithm = treatment_algorithm,
      outcome_algorithm = outcome_algorithm,
      weight_name = weight_name,
      n_folds = n_folds,
      folds_name = folds_name
    )
  }

  ##############################################
  # Estimate the factual means and disparities #
  ##############################################

  if (is.null(weight_name)) {
    factual_means <- data %>%
    dplyr::group_by(dplyr::across(tidyselect::matches(paste0("^",category_name,"$")))) %>%
      dplyr::summarize(dplyr::across(tidyselect::matches(paste0("^",outcome_name,"$")), mean),
                       .groups = "drop") %>%
      dplyr::rename_with(function(x) dplyr::case_when(x == outcome_name ~ "estimate",
                                                      T ~ x))
  } else {
    factual_means <- data %>%
      dplyr::mutate(gapclosing.weight = data[[weight_name]]) %>%
      dplyr::group_by(dplyr::across(tidyselect::matches(paste0("^",category_name,"$")))) %>%
      dplyr::summarize(dplyr::across(tidyselect::matches(paste0("^",outcome_name,"$")), stats::weighted.mean, w = gapclosing.weight),
                       .groups = "drop") %>%
      dplyr::rename_with(function(x) dplyr::case_when(x == outcome_name ~ "estimate",
                                                      T ~ x))
  }


  factual_disparities <- pairwise_diff(factual_means, category_name = category_name)

  ####################################################
  # Calculate changes from factual to counterfactual #
  ####################################################

  # Write a function that will be applied separately to means and disparities
  calculate_changes <- function(factual_estimates, counterfactual_estimates) {
    counterfactual_estimates %>%
      dplyr::rename(c = estimate) %>%
      dplyr::left_join(factual_estimates %>%
                         dplyr::select(tidyselect::all_of(c(category_name,"estimate"))) %>%
                         dplyr::rename(f = estimate),
                       by = category_name) %>%
      dplyr::mutate(additive = f - c,
                    proportional = (f - c) / f) %>%
      dplyr::select(tidyselect::all_of(c(category_name,"method","additive","proportional"))) %>%
      tidyr::pivot_longer(cols = c("additive","proportional"),
                          names_to = "change_type",
                          values_to = "estimate") %>%
      dplyr::mutate(change_formula = dplyr::case_when(change_type == "additive" ~ "f-c",
                                                      change_type == "proportional" ~ "(f-c)/f")) %>%
      dplyr::select(tidyselect::all_of(c(category_name,"method","change_type","change_formula","estimate")))
  }

  change_means <- calculate_changes(factual_estimates = factual_means,
                                    counterfactual_estimates = counterfactual_point$counterfactual_means)

  change_disparities <- calculate_changes(factual_estimates = factual_disparities,
                                          counterfactual_estimates = counterfactual_point$counterfactual_disparities)

  ##############################
  # Create an object to return #
  ##############################

  # Identify the primary method to return. Others will be returned as alternatives.
  primary_estimator_name <- dplyr::case_when(!is.null(treatment_formula) & !is.null(outcome_formula) ~ "doubly_robust",
                                             !is.null(outcome_formula) ~ "outcome_modeling",
                                             !is.null(treatment_formula) ~ "treatment_modeling")

  # Organize results by estimator instead of by estimand
  all_estimators <- lapply(
    c(outcome_modeling = "outcome_modeling",
      treatment_modeling = "treatment_modeling",
      doubly_robust = "doubly_robust"),
    function(method_case) {
      lapply(
        list(counterfactual_means = counterfactual_point$counterfactual_means,
             counterfactual_disparities = counterfactual_point$counterfactual_disparities,
             change_means = change_means,
             change_disparities = change_disparities),
        function(x) {
          x %>%
            dplyr::filter(method == method_case) %>%
            dplyr::select(-method)
        }
      )
    }
  )

  # Prepare a list to return
  gapclosing.no.se <- c(list(factual_means = factual_means,
                             factual_disparities = factual_disparities),
                        all_estimators[[primary_estimator_name]],
                        list(all_estimators = all_estimators,
                             primary_estimator_name = primary_estimator_name),
                        treatment_model = list(counterfactual_point$treatment_model),
                        outcome_model = list(counterfactual_point$outcome_model),
                        call = match.call(),
                        arguments = list(arguments))
  class(gapclosing.no.se) <- "gapclosing"

  if (!se) {
    return(gapclosing.no.se)
  }

  # If se = TRUE, the next bit adds the standard errors before returning
  # by repeatedly calling this function with a bootstrap sample and se = FALSE.

  ##################################################################
  # Bootstrap standard errors by recursively calling this function #
  ##################################################################
  if (se) {
    # Prepare to run bootstrap in parallel if requested by the user
    if (!is.null(parallel_cores)) {
      cl <- parallel::makeCluster(parallel_cores)
      doParallel::registerDoParallel(cl)
      `%domethod%` <- `%dorng%`
    } else {
      `%domethod%` <- `%do%`
    }
    # Produce standard errors by bootstrapping
    bs_estimates <- foreach::foreach(i = 1:bootstrap_samples, .combine = "rbind", .packages = "dplyr") %domethod% {
      if (i %% bootstrap_samples == 100) {
        print(paste("Beginning draw",i,"of",bootstrap_samples))
      }
      if (bootstrap_method == "simple") {
        data_star <- data %>%
          dplyr::slice_sample(prop = 1, replace = TRUE)
      } else if (bootstrap_method == "stratified") {
        # This is stratified sampling within category x treatment.
        # This ensures that every cell contains data and avoids yielding se = NA
        # if some of the cells do not contain data in some samples.
        data_star <- data %>%
          dplyr::group_by(dplyr::across(tidyselect::all_of(c(category_name,treatment_name)))) %>%
          dplyr::slice_sample(prop = 1, replace = TRUE) %>%
          dplyr::ungroup()
      }
      gapclosing_star <- gapclosing(data = data_star,
                                    outcome_formula = outcome_formula,
                                    treatment_name = treatment_name,
                                    treatment_formula = treatment_formula,
                                    outcome_name = outcome_name,
                                    category_name = category_name,
                                    counterfactual_assignments = counterfactual_assignments,
                                    weight_name = weight_name,
                                    se = FALSE,
                                    treatment_algorithm = treatment_algorithm,
                                    outcome_algorithm = outcome_algorithm,
                                    sample_split = sample_split,
                                    n_folds = n_folds,
                                    folds_name = folds_name)
      return(as.data.frame(gapclosing_star))
    }
    if (!is.null(parallel_cores)) {
      parallel::stopCluster(cl)
      rm(cl)
    }

    results.df <- bs_estimates %>%
      dplyr::group_by(dplyr::across(tidyselect::all_of(c(category_name, "estimand", "estimator", "primary")))) %>%
      dplyr::summarize(se = stats::sd(estimate),
                       .groups = "drop") %>%
      dplyr::left_join(as.data.frame(gapclosing.no.se),
                       by = c(category_name,"estimand","estimator","primary")) %>%
      dplyr::select(tidyselect::all_of(c(category_name, "estimand", "estimator", "primary", "estimate", "se"))) %>%
      dplyr::mutate(ci.min = estimate - stats::qnorm(.975) * se,
                    ci.max = estimate + stats::qnorm(.975) * se)

    results.list <- df_to_gapclosing_list(results.df)
    gapclosing.with.se <- c(results.list,
                            gapclosing.no.se[!names(gapclosing.no.se) %in% names(results.list)])
    class(gapclosing.with.se) <- "gapclosing"
    return(gapclosing.with.se)
  }
}

test_that("Outcome modeling recovers known answer", {
  sim.data <- data.frame(category = c(1,1,1,1,2,2,2,2),
                         # Treat half of each category
                         treatment = c(0,0,1,1,0,0,1,1),
                         # Define outcomes to have
                         # a gap of 2 under treatment 0
                         # a gap of 0 under treatment 1
                         outcome = c(0,0,1,1,2,2,1,1))
  estimate_outcome_0 <- gapclosing(sim.data,
                                   counterfactual_assignments = 0,
                                   outcome_formula = formula(outcome ~ category*treatment),
                                   category_name = "category",
                                   treatment_name = "treatment")
  estimate_outcome_1 <- gapclosing(sim.data,
                                   counterfactual_assignments = 1,
                                   outcome_formula = formula(outcome ~ category*treatment),
                                   category_name = "category",
                                   treatment_name = "treatment")
  expect_equal(sort(abs(estimate_outcome_0$counterfactual_means$estimate)), c(0,2))
  expect_equal(sort(abs(estimate_outcome_1$counterfactual_means$estimate)), c(1,1))
  expect_equal(abs(estimate_outcome_0$counterfactual_disparities$estimate), c(2,2))
  expect_equal(abs(estimate_outcome_1$counterfactual_disparities$estimate), c(0,0))
})

test_that("Treatment modeling recovers known answer", {
  sim.data <- data.frame(category = c(1,1,1,1,2,2,2,2),
                         # Treat half of each category
                         treatment = c(0,0,1,1,0,0,1,1),
                         # Define outcomes to have
                         # a gap of 2 under treatment 0
                         # a gap of 0 under treatment 1
                         outcome = c(0,0,1,1,2,2,1,1))
  estimate_outcome_0 <- gapclosing(sim.data,
                                   counterfactual_assignments = 0,
                                   treatment_formula = formula(treatment ~ category),
                                   category_name = "category",
                                   outcome_name = "outcome")
  estimate_outcome_1 <- gapclosing(sim.data,
                                   counterfactual_assignments = 1,
                                   treatment_formula = formula(treatment ~ category),
                                   category_name = "category",
                                   outcome_name = "outcome")
  expect_equal(sort(abs(estimate_outcome_0$counterfactual_means$estimate)), c(0,2))
  expect_equal(sort(abs(estimate_outcome_1$counterfactual_means$estimate)), c(1,1))
  expect_equal(abs(estimate_outcome_0$counterfactual_disparities$estimate), c(2,2))
  expect_equal(abs(estimate_outcome_1$counterfactual_disparities$estimate), c(0,0))
})

test_that("Doubly robust recovers known answer", {
  sim.data <- data.frame(category = c(1,1,1,1,2,2,2,2),
                         # Treat half of each category
                         treatment = c(0,0,1,1,0,0,1,1),
                         # Define outcomes to have
                         # a gap of 2 under treatment 0
                         # a gap of 0 under treatment 1
                         outcome = c(0,0,1,1,2,2,1,1))
  estimate_outcome_0 <- gapclosing(sim.data,
                                   counterfactual_assignments = 0,
                                   outcome_formula = formula(outcome ~ category*treatment),
                                   treatment_formula = formula(treatment ~ category),
                                   category_name = "category")
  estimate_outcome_1 <- gapclosing(sim.data,
                                   counterfactual_assignments = 1,
                                   outcome_formula = formula(outcome ~ category*treatment),
                                   treatment_formula = formula(treatment ~ category),
                                   category_name = "category")
  expect_equal(sort(abs(estimate_outcome_0$counterfactual_means$estimate)), c(0,2))
  expect_equal(sort(abs(estimate_outcome_1$counterfactual_means$estimate)), c(1,1))
  expect_equal(abs(estimate_outcome_0$counterfactual_disparities$estimate), c(2,2))
  expect_equal(abs(estimate_outcome_1$counterfactual_disparities$estimate), c(0,0))
})

test_that("Test weighted estimation", {
  sim.data <- data.frame(category = c(1,1,1,1,2,2,2,2),
                         # Treat half of each category
                         treatment = c(0,0,1,1,0,0,1,1),
                         # Define outcomes to have
                         # a gap of 2 under treatment 0
                         # a gap of 0 under treatment 1
                         outcome = c(0,0,1,1,2,2,1,1),
                         # Triple weight on untreated cases
                         my_weight = c(3,3,1,1,3,3,1,1))
  estimate_outcome_0 <- gapclosing(sim.data,
                                   counterfactual_assignments = 0,
                                   outcome_formula = formula(outcome ~ category*treatment),
                                   category_name = "category",
                                   treatment_name = "treatment",
                                   weight_name = "my_weight")
  estimate_outcome_1 <- gapclosing(sim.data,
                                   counterfactual_assignments = 1,
                                   outcome_formula = formula(outcome ~ category*treatment),
                                   category_name = "category",
                                   treatment_name = "treatment",
                                   weight_name = "my_weight")
  # Check that factual disparities correctly use weights
  expect_equal(abs(estimate_outcome_0$factual_disparities$estimate), c(1.5,1.5))
  expect_equal(abs(estimate_outcome_1$factual_disparities$estimate), c(1.5,1.5))
  # Check that counterfactual disparity is still 2 in treatment 0 and 0 in treatment 1
  expect_equal(abs(estimate_outcome_0$counterfactual_disparities$estimate), c(2,2))
  expect_equal(abs(estimate_outcome_1$counterfactual_disparities$estimate), c(0,0))

  # Test in a setting where the weights vary within (category x treatment)
  sim.data <- data.frame(category = c(1,1,1,1,2,2,2,2),
                         # Treat half of each category
                         treatment = c(0,0,1,1,0,0,1,1),
                         # Define outcomes to have
                         # a gap of 2 under treatment 0
                         # a gap of 0 under treatment 1
                         outcome = c(0,4,0,8,0,4,0,4),
                         # Triple weight on untreated cases
                         my_weight = c(1,3,1,3,1,3,1,3))
  estimate_outcome_0 <- gapclosing(sim.data,
                                   counterfactual_assignments = 0,
                                   outcome_formula = formula(outcome ~ category*treatment),
                                   category_name = "category",
                                   treatment_name = "treatment",
                                   weight_name = "my_weight")
  expect_equal(sort(estimate_outcome_0$counterfactual_means$estimate), c(3,3))
  estimate_outcome_1 <- gapclosing(sim.data,
                                   counterfactual_assignments = 1,
                                   outcome_formula = formula(outcome ~ category*treatment),
                                   category_name = "category",
                                   treatment_name = "treatment",
                                   weight_name = "my_weight")
  expect_equal(sort(estimate_outcome_1$counterfactual_means$estimate), c(3,6))
})

test_that("Test that results hold with arbitrary names for the variables", {
  sim.data <- data.frame(arbitrary_category = c(1,1,1,1,2,2,2,2),
                         # Treat half of each category
                         arbitrary_treatment = c(0,0,1,1,0,0,1,1),
                         irrelevant_covariate = c(0,1,0,1,0,1,0,1),
                         # Define outcomes to have
                         # a gap of 2 under treatment 0
                         # a gap of 0 under treatment 1
                         my_outcome = c(0,0,1,1,2,2,1,1),
                         # Throw in variables named treatment, category, outcome,
                         # which should be just ignored by the function if all is working.
                         treatment = rep(1,8),
                         category = rep(1,8),
                         outcome = rep(1,8))
  estimate_outcome_0 <- gapclosing(sim.data,
                                   counterfactual_assignments = 0,
                                   outcome_formula = formula(my_outcome ~ arbitrary_category*arbitrary_treatment + irrelevant_covariate),
                                   treatment_formula = formula(arbitrary_treatment ~ arbitrary_category + irrelevant_covariate),
                                   category_name = "arbitrary_category")
  estimate_outcome_1 <- gapclosing(sim.data,
                                   counterfactual_assignments = 1,
                                   outcome_formula = formula(my_outcome ~ arbitrary_category*arbitrary_treatment + irrelevant_covariate),
                                   treatment_formula = formula(arbitrary_treatment ~ arbitrary_category + irrelevant_covariate),
                                   category_name = "arbitrary_category")
  expect_equal(sort(abs(estimate_outcome_0$counterfactual_means$estimate)), c(0,2))
  expect_equal(sort(abs(estimate_outcome_1$counterfactual_means$estimate)), c(1,1))
  expect_equal(abs(estimate_outcome_0$counterfactual_disparities$estimate), c(2,2))
  expect_equal(abs(estimate_outcome_1$counterfactual_disparities$estimate), c(0,0))
})

#' Pairwise difference calculator
#' @description A function to estimate the pairwise differences of estimates made for each category.
#' @param category_means_data Data frame containing two columns: the category-specific mean estimate (a column named \code{estimate}) and the category name (named as specified in \code{category_name})
#' @param category_name The name of the column containing the category identifier.
#' @return A data frame with pairwise differences of \code{estimate} over pairs of categories.
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @references Friedman J, Hastie T, Tibshirani R (2010). "Regularization Paths for Generalized Linear Models via Coordinate Descent." Journal of Statistical Software, 33(1), 1–22. {https://www.jstatsoft.org/htaccess.php?volume=33&type=i&issue=01}.
#' @references Wood S (2017). Generalized Additive Models: An Introduction with R, 2 edition. Chapman and Hall/CRC.
#' @references Wright MN, Ziegler A (2017). "ranger: A Fast Implementation of Random Forests for High Dimensional Data in C++ and R." Journal of Statistical Software, 77(1), 1–17. doi: 10.18637/jss.v077.i01.
#' @importFrom foreach %do%
#' @importFrom magrittr %>%
#' @export
#'
#' @examples
#' sim_data <- data.frame(example_category = c("A","B","C"),
#'                        estimate = c(1,2,3))
#' pairwise_diff(sim_data, category_name = "example_category")

pairwise_diff <- function(category_means_data, category_name) {

  # Check arguments
  if (!("estimate" %in% colnames(category_means_data))) {
    stop("Error: category_means_data should have a column named estimate containing the mean outcome in that category")
  }
  if (!(category_name %in% colnames(category_means_data))) {
    stop("Error: category_name should be a character string with the name of a column in category_means_data")
  }

  # Prepare for non-standard evaluation
  category1 <- category2 <- category <- category1_value <- category2_value <- estimate <- NULL

  # Prepare a data frame with the category differences
  result <- category_means_data %>%
    # Create a data frame with all category values
    dplyr::select(tidyselect::matches(paste0("^",category_name,"$"))) %>%
    # Call that category1
    dplyr::rename_with(function(x) "category1") %>%
    # Sort that data frame
    dplyr::arrange(category1) %>%
    # Create a second category value identical to the first
    dplyr::mutate(category2 = category1) %>%
    # Expand to have all combinations of the two categories
    tidyr::expand(category1,category2) %>%
    # Remove cases where both categories are the same
    dplyr::filter(category1 != category2) %>%
    # Merge in the factual mean for category 1
    dplyr::left_join(category_means_data %>%
                       dplyr::rename(category1 = category_name,
                                     category1_value = estimate),
                     by = "category1") %>%
    # Merge in the factual mean for category 2
    dplyr::left_join(category_means_data %>%
                       dplyr::rename(category2 = category_name,
                                     category2_value = estimate),
                     by = "category2") %>%
    # Produce a difference estimate and give it a name
    dplyr::mutate(difference_name = paste(category1,"-",category2),
                  estimate = category1_value - category2_value) %>%
    # Rename to make the output align with the input
    dplyr::rename_with(function(x) dplyr::case_when(x == "difference_name" ~ category_name,
                                                    x == "estimate" ~ "estimate",
                                                    T ~ x)) %>%
    dplyr::select(tidyselect::all_of(c(category_name, "estimate")))

  return(result)
}

#' Plot a disparity
#' @description Plots the factual and counterfactual mean outcomes in two categories. The returned object is a ggplot2 object which can be further customized using the syntax of ggplot2.
#' @param x An object of class \code{gapclosing}, which results from a call to the function \code{gapclosing}
#' @param category_A The first category to be plotted. A value of the category_name variable in x.
#' @param category_B The second category to be plotted. Must be a value of \code{x$category}
#' @param custom_ylab Custom y-axis label. Defaults to "Mean Outcome"
#' @param custom_xlab Custom x-axis label. Defaults to "Category"
#' @return A \code{ggplot2} object
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @importFrom magrittr %>%
#' @export

disparityplot <- function(x, category_A, category_B, custom_ylab = "Mean Outcome", custom_xlab = "Category") {
  # Initialize non-standard evaluation variables to avoid R CMD check warnings.
  gapclosing.category <- setting <- estimate <- se <- label <- y <- Factual <- Counterfactual <- NULL
  # Create data frame for plot
  forplot <- x$factual_means %>%
    dplyr::bind_rows(x$factual_disparities) %>%
    # Label as the factual values
    dplyr::mutate(setting = "Factual") %>%
    # Append rows for the counterfactual values
    dplyr::bind_rows(x$counterfactual_means %>%
                       dplyr::bind_rows(x$counterfactual_disparities) %>%
                       # Label as the counterfactual values
                       dplyr::mutate(setting = "Counterfactual")) %>%
    # Rename the category variable to the name category
    dplyr::rename_with(function(old_name) ifelse(old_name == x$arguments$category_name, "gapclosing.category", old_name)) %>%
    # Restrict to the category contrast of interest
    dplyr::filter(gapclosing.category %in% c(category_A, category_B, paste(category_A,"-",category_B))) %>%
    # Place categories in the user-specified order for plotting
    dplyr::mutate(gapclosing.category = forcats::fct_relevel(gapclosing.category, category_A, category_B, paste(category_A,"-",category_B)),
                  # Reverse the order of setting so that factual appears before counterfactual
                  setting = forcats::fct_rev(setting))

  # Calculate means for the plot
  means <- forplot %>%
    dplyr::filter(!grepl("-",gapclosing.category))
  disparities <- forplot %>%
    dplyr::filter(grepl("-",gapclosing.category))

  plot <- means %>%
    ggplot2::ggplot() +
    ggplot2::geom_point(ggplot2::aes(color = setting, shape = setting,
                                     x = gapclosing.category, y = estimate),
                        position = ggplot2::position_dodge(width = .1)) +
    ggplot2::geom_line(ggplot2::aes(color = setting,
                                    x = ifelse(setting == "Factual", 1.25, 1.75),
                                    y = estimate),
                       position = ggplot2::position_dodge(width = .1),
                       linewidth = .5) +
    ggplot2::geom_segment(ggplot2::aes(color = setting,
                                       x = gapclosing.category, xend = ifelse(setting == "Factual", 1.25, 1.75),
                                       y = estimate, yend = estimate),
                          linetype = "dashed",
                          position = ggplot2::position_dodge(width = .1),
                          linewidth = .5) +
    ggplot2::geom_text(data = disparities,
                       ggplot2::aes(color = setting,
                                    x = ifelse(setting == "Factual", 1.25, 1.75),
                                    label = paste0(ifelse(setting == "Factual","Descriptive\nDisparity","Gap Closing\nEstimand")),
                                    y = max(means$estimate) + .2 * (max(means$estimate) - min(means$estimate))),
                       vjust = 0,
                       size = 3,
                       show.legend = FALSE) +
    ggplot2::geom_text(data = means %>%
                         dplyr::group_by(setting) %>%
                         dplyr::summarize(label = format(abs(diff(estimate)), digits = 2),
                                          y = mean(estimate)),
                       ggplot2::aes(color = setting,
                                    x = ifelse(setting == "Factual", 1.25, 1.75),
                                    label = label,
                                    y = y, hjust = ifelse(setting == "Factual", -.2, 1.2)),
                       position = ggplot2::position_dodge(width = .1),
                       size = 3,
                       show.legend = FALSE) +
    # Note the treatment effects
    ggplot2::geom_segment(data = means %>%
                            dplyr::select(setting, gapclosing.category, estimate) %>%
                            tidyr::spread(key = setting, value = estimate) %>%
                            dplyr::mutate(x = ifelse(gapclosing.category == category_A, .8, 2.2)),
                          ggplot2::aes(x = x, xend = x, y = Factual, yend = Counterfactual),
                          arrow = grid::arrow(length = grid::unit(.1,"in")),
                          color = "gray") +
    ggplot2::geom_text(data = means %>%
                         dplyr::group_by(gapclosing.category) %>%
                         dplyr::summarize(
                           y = mean(estimate),
                           .groups = "drop"
                         ) %>%
                         dplyr::mutate(x = ifelse(gapclosing.category == category_A, .8, 2.2)),
                       ggplot2::aes(x = x, y = y, vjust = ifelse(gapclosing.category == category_A,-1,2)),
                       label = "Causal Effect", color = "gray",
                       size = 2, angle = 90) +
    # Edit scale appearances
    ggplot2::scale_x_discrete(name = custom_xlab,
                              expand = ggplot2::expansion(add = .35)) +
    ggplot2::scale_y_continuous(name = custom_ylab,
                                expand = ggplot2::expansion(mult = .2)) +
    ggplot2::theme_bw() +
    ggplot2::theme(legend.position = "right",
                   legend.title = ggplot2::element_blank())

  # If there was a standard error, add the CI to the plot
  if (x$arguments$se) {
    plot <- plot +
      ggplot2::geom_errorbar(ggplot2::aes(color = setting,
                                          x = gapclosing.category,
                                          ymin = estimate - stats::qnorm(.975) * se,
                                          ymax = estimate + stats::qnorm(.975) * se),
                             width = .1, size = .3,
                             position = ggplot2::position_dodge(width = .1))
  }

  return(plot)
}

#' Point estimator for gap-closing estimands
#' @description This is an internal function typically called from other functions rather than by the user. It uses a learning sample to learn the nuisance functions (treatment and outcome model) and then an auxiliary estimation sample to use those functions in estimation of the gap-closing estimand. For single-sample estimation, both the learning and estimation samples are the same. For cross-fitting, this function is called repeatedly with the roles of each dataset swapped.
#' @param data_learn Data frame in which treatment and outcome models will be learned
#' @param data_estimate Data frame in which the learned models will be converted to an estimate of the gap-closing estimand
#' @param counterfactual_assignments Numeric scalar or vector of length nrow(data), each element of which is on the \[0,1\] interval. If a scalar, the counterfactual probability by which all units are assigned to treatment condition 1. If a vector, each element i corresponds to the counterfactual probability by which each unit i is assigned to treatment condition 1.
#' @param outcome_formula Model formula the outcome. Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph). If \code{outcome_algorithm} = "ranger", then the outcome model will be fit separately on the treatment and control groups. Otherwise, the user must specify all interactions in the formula.
#' @param treatment_formula Treatment formula, in the style formula(treatment ~ covariates). Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph).
#' @param category_name Character name of the variable indicating the categories over which the gap is defined. Must be the name of a column in \code{data}.
#' @param outcome_name Character name of the outcome variable. Only required when there is no outcome_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.
#' @param treatment_name Character name of the treatment variable. Only required when there is no treatment_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.
#' @param treatment_algorithm Character name of the algorithm for the treatment model. One of "glm", "ridge", "gam", or "ranger". Defaults to "glm", which is a logit model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}). If "ranger", this function avoids propensity scores equal to 0 or 1 by bottom- and top-coding predicted values at .001 and .999.
#' @param outcome_algorithm Character name of the algorithm for the outcome model. One of "lm", "ridge", "gam", or "ranger". Defaults to "lm", which is an OLS model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}).
#' @param weight_name Character name of a sampling weight variable, if any, which captures the inverse probability of inclusion in the sample. The default assumes a simple random sample (all weights equal).
#' @return @return A list with four elements.
#' \itemize{
#' \code{counterfactual_means} A tibble with a counterfactual mean estimate for each category
#' \code{counterfactual_means} A tibble with a counterfactual disparity estimate for each pair of categories
#' \code{treatment_model} Object containing the fitted treatment model
#' \code{outcome_model} Object containing the fitted outcome model
#' }
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @importFrom foreach %do%
#' @importFrom magrittr %>%
#' @export

point_estimator <- function(
  data_learn,
  data_estimate,
  counterfactual_assignments,
  outcome_formula,
  treatment_formula,
  category_name,
  outcome_name,
  treatment_name,
  treatment_algorithm = "glm",
  outcome_algorithm = "lm",
  weight_name = NULL
) {
  # Initialize non-standard evaluation variables to avoid R CMD check warnings.
  gapclosing.weight <- gapclosing.treatment <- gapclosing.outcome <- gapclosing.counterfactual_assignments <-
    gapclosing.m_fitted <- gapclosing.pi_i <- gapclosing.m_i <- gapclosing.w_i <- gapclosing.yhat <-
    gapclosing.yhat1 <- gapclosing.yhat0 <- gapclosing.residual <- outcome_modeling <-
    gapclosing.robust_augmentation <- method <- method_case <- NULL
  # Assign a weight variable
  if (is.null(weight_name)) {
    data_learn$gapclosing.weight <- rep(1,nrow(data_learn))
    data_estimate$gapclosing.weight <- rep(1,nrow(data_estimate))
  } else {
    data_learn$gapclosing.weight <- data_learn[[weight_name]]
    data_estimate$gapclosing.weight <- data_estimate[[weight_name]]
  }
  # Enforce normalization of the weights
  data_learn$gapclosing.weight <-  data_learn$gapclosing.weight / mean(data_learn$gapclosing.weight, na.rm = TRUE)
  data_estimate$gapclosing.weight <-  data_estimate$gapclosing.weight / mean(data_estimate$gapclosing.weight, na.rm = TRUE)
  # Create easily-accessible treatment and outcome variables with known names,
  # as well as containing the counterfactual assignments
  data_learn <- data_learn %>%
    dplyr::mutate(gapclosing.treatment = data_learn[[treatment_name]],
                  gapclosing.outcome = data_learn[[outcome_name]],
                  gapclosing.counterfactual_assignments = NA)
  data_estimate <- data_estimate %>%
    dplyr::mutate(gapclosing.treatment = data_estimate[[treatment_name]],
                  gapclosing.outcome = data_estimate[[outcome_name]],
                  gapclosing.counterfactual_assignments = counterfactual_assignments)

  #################################################
  # Prepare datasets in which to make predictions #
  #################################################

  # Prepare data modified to take each counterfactual treatment value,
  # to be used to predict potential outcomes
  data_estimate_1 <- data_estimate_0 <- data_estimate
  data_estimate_1[[treatment_name]] <- 1
  data_estimate_0[[treatment_name]] <- 0

  #######################################################
  # Predict the treatment probability given confounders #
  #######################################################
  if (is.null(treatment_formula)) {
    fit_m <- NULL
    data_estimate <- data_estimate %>%
      dplyr::mutate(gapclosing.m_fitted = NA)
  } else if (treatment_algorithm == "glm") {
    # The withCallingHandlers turns off the warning for non-integer number of successes,
    # since we expect that warning when there are weights.
    fit_m <- withCallingHandlers({
      stats::glm(treatment_formula,
                 data = data_learn,
                 family = stats::binomial,
                 weights = gapclosing.weight)
    }, warning = function(w) {
      if (grepl("non-integer",conditionMessage(w)))
        invokeRestart("muffleWarning")
    })
    # Extract fitted propensity score
    data_estimate <- data_estimate %>%
      dplyr::mutate(gapclosing.m_fitted = stats::predict(fit_m, newdata = data_estimate, type = "response"))
  } else if (treatment_algorithm == "ridge") {
    fit_ridge.out <- fit_ridge(data = data_learn,
                               model_formula = treatment_formula,
                               to_predict = data_estimate)
    fit_m <- fit_ridge.out$fit
    data_estimate <- data_estimate %>%
      dplyr::mutate(gapclosing.m_fitted = fit_ridge.out$fitted)
  } else if (treatment_algorithm == "gam") {
    fit_m <- withCallingHandlers({
      mgcv::gam(treatment_formula,
                data = data_learn,
                family = stats::binomial,
                weights = gapclosing.weight)
    }, warning = function(w) {
      if (grepl("non-integer",conditionMessage(w)))
        invokeRestart("muffleWarning")
    })
    # Extract fitted propensity score
    data_estimate <- data_estimate %>%
      dplyr::mutate(gapclosing.m_fitted = mgcv::predict.gam(fit_m, newdata = data_estimate, type = "response"))
  } else if (treatment_algorithm == "ranger") {
    fit_m <- ranger::ranger(treatment_formula,
                            data = data_learn,
                            case.weights = data_learn$gapclosing.weight)
    data_estimate <- data_estimate %>%
      dplyr::mutate(gapclosing.m_fitted = stats::predict(fit_m, data = data_estimate)$predictions,
                    # Truncate the fitted propensity score because random forest can
                    # produce fits of exactly 0 and 1, which create weighting problems
                    gapclosing.m_fitted = dplyr::case_when(gapclosing.m_fitted < .001 ~ .001,
                                                           gapclosing.m_fitted <= .999 ~ gapclosing.m_fitted,
                                                           gapclosing.m_fitted > .999 ~ .999))
    # To keep data sets in parallel to each other, make NA for that variable in data_learn
    data_learn$gapclosing.m_fitted <- NA
  }

  ############################################################
  # Predict the outcome mean given treatment and confounders #
  ############################################################

  if (is.null(outcome_formula)) {
    fit_g <- NULL
    data_estimate <- data_estimate %>%
      dplyr::mutate(gapclosing.yhat1 = NA,
                    gapclosing.yhat0 = NA,
                    gapclosing.yhat = NA,
                    gapclosing.residual = NA)
  } else if (outcome_algorithm == "lm") {
    fit_g <- stats::lm(outcome_formula,
                       data = data_learn,
                       weights = gapclosing.weight)
    data_estimate <- data_estimate %>%
      dplyr::mutate(gapclosing.yhat1 = stats::predict(fit_g, newdata = data_estimate_1),
                    gapclosing.yhat0 = stats::predict(fit_g, newdata = data_estimate_0),
                    gapclosing.yhat = dplyr::case_when(gapclosing.treatment == 1 ~ gapclosing.yhat1,
                                            gapclosing.treatment == 0 ~ gapclosing.yhat0),
                    gapclosing.residual = gapclosing.outcome - gapclosing.yhat)
  } else if (outcome_algorithm == "ridge") {
    fit_ridge.out <- fit_ridge(data = data_learn,
                               model_formula = outcome_formula,
                               to_predict = rbind(data_estimate_1,data_estimate_0))
    fit_g <- fit_ridge.out
    data_estimate <- data_estimate %>%
      dplyr::mutate(gapclosing.yhat1 = fit_ridge.out$fitted[1:nrow(data_estimate_1)],
                    gapclosing.yhat0 = fit_ridge.out$fitted[(nrow(data_estimate_1) + 1):(nrow(data_estimate_1) + nrow(data_estimate_0))],
                    gapclosing.yhat = dplyr::case_when(gapclosing.treatment == 1 ~ gapclosing.yhat1,
                                                       gapclosing.treatment == 0 ~ gapclosing.yhat0),
                    gapclosing.residual = gapclosing.outcome - gapclosing.yhat)
  } else if (outcome_algorithm == "gam") {
    fit_g <- mgcv::gam(outcome_formula,
                       data = data_learn,
                       weights = gapclosing.weight)
    data_estimate <- data_estimate %>%
      dplyr::mutate(gapclosing.yhat1 = stats::predict(fit_g, newdata = data_estimate_1),
                    gapclosing.yhat0 = stats::predict(fit_g, newdata = data_estimate_0),
                    gapclosing.yhat = dplyr::case_when(gapclosing.treatment == 1 ~ gapclosing.yhat1,
                                            gapclosing.treatment == 0 ~ gapclosing.yhat0),
                    gapclosing.residual = gapclosing.outcome - gapclosing.yhat)
  } else if (outcome_algorithm == "ranger") {
    data_learn_0 <- data_learn[data_learn[[treatment_name]] == 0,]
    data_learn_1 <- data_learn[data_learn[[treatment_name]] == 1,]
    if (!all(colnames(data_learn_0) == colnames(data_estimate)) |
        !all(colnames(data_learn_1) == colnames(data_estimate))) {
      stop("Error: data_learn_0, data_learn_1, and data_estimate should have the same columns")
    }
    fit_g_1 <- ranger::ranger(outcome_formula,
                              data = data_learn_1,
                              case.weights = data_learn_1$gapclosing.weight)
    fit_g_0 <- ranger::ranger(outcome_formula,
                              data = data_learn_0,
                              case.weights = data_learn_0$gapclosing.weight)
    fit_g <- list(fit_g_1 = fit_g_1, fit_g_0 = fit_g_0)
    data_estimate <- data_estimate %>%
      dplyr::mutate(gapclosing.yhat1 = stats::predict(fit_g_1, data = data_estimate)$predictions,
                    gapclosing.yhat0 = stats::predict(fit_g_0, data = data_estimate)$predictions,
                    gapclosing.yhat = dplyr::case_when(gapclosing.treatment == 1 ~ gapclosing.yhat1,
                                            gapclosing.treatment == 0 ~ gapclosing.yhat0),
                    gapclosing.residual = gapclosing.outcome - gapclosing.yhat)
  }

  ##########################
  # Aggregate to estimates #
  ##########################

  counterfactual_means <- data_estimate %>%
    # Create the propensity score
    dplyr::mutate(gapclosing.m_i = ifelse(gapclosing.treatment == 1,
                               as.numeric(gapclosing.m_fitted),
                               1 - as.numeric(gapclosing.m_fitted))) %>%
    # Create the counterfactual probability of the factual treatment
    dplyr::mutate(gapclosing.pi_i = dplyr::case_when(gapclosing.treatment == 1 ~ gapclosing.counterfactual_assignments,
                                          gapclosing.treatment == 0 ~ 1 - gapclosing.counterfactual_assignments)) %>%
    # Create the weight that combines all of those
    dplyr::mutate(gapclosing.w_i = gapclosing.weight * gapclosing.pi_i / gapclosing.m_i) %>%
    # Group by category
    dplyr::group_by(dplyr::across(tidyselect::matches(paste0("^",category_name,"$")))) %>%
    # Take weighted means to produce estimates
    dplyr::summarize(outcome_modeling = stats::weighted.mean(gapclosing.yhat1 * gapclosing.counterfactual_assignments +
                                                               gapclosing.yhat0 * (1 - gapclosing.counterfactual_assignments),
                                                             w = gapclosing.weight),
                     treatment_modeling = stats::weighted.mean(gapclosing.outcome,
                                                               w = gapclosing.w_i),
                     gapclosing.robust_augmentation = stats::weighted.mean(-gapclosing.residual,
                                                                w = gapclosing.w_i),
                     .groups = "drop") %>%
    dplyr::mutate(doubly_robust = outcome_modeling - gapclosing.robust_augmentation) %>%
    dplyr::select(-gapclosing.robust_augmentation) %>%
    tidyr::pivot_longer(c("outcome_modeling","treatment_modeling","doubly_robust"),
                        names_to = "method",
                        values_to = "estimate")

  # Difference those to produce gap-closing estimates
  counterfactual_disparities <- foreach::foreach(method_case = unique(counterfactual_means$method), .combine = "rbind") %do% {
    pairwise_diff(category_means_data = counterfactual_means %>%
                    dplyr::filter(method == method_case) %>%
                    dplyr::select(-method),
                  category_name = category_name) %>%
      dplyr::mutate(method = method_case)
  }

  object_to_return <- list(counterfactual_means = counterfactual_means,
                           counterfactual_disparities = counterfactual_disparities,
                           treatment_model = fit_m,
                           outcome_model = fit_g)

  return(object_to_return)
}

#' Summary function for gapclosing objects
#' @description Summarizes the S3 class object returned by the gapclosing function
#' @param object An object of class \code{gapclosing}, which results from a call to the function \code{gapclosing}
#' @param ... Other arguments to \code{summary} commands
#' @return Prints a summary of the estimates.
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @export

summary.gapclosing <- function(object, ...) {

  # Prepare for non-standard evaluation
  treatment_name <- NULL

  if (is.null(object$arguments$treatment_name)) {
    object$arguments$treatment_name <- all.vars(object$arguments$treatment_formula)[1]
  }
  if (object$arguments$sample_split == "single_sample") {
    cat(paste("Gap-closing estimates using",object$primary_estimator_name,"estimation on one sample.\n"))
  } else {
    cat(paste("Gap-closing estimates using",object$primary_estimator_name,"estimation with cross-fitting.\n"))
  }
  if (object$primary_estimator_name %in% c("treatment_modeling","doubly_robust")) {
    cat(paste("\nTreatment model was",object$arguments$treatment_algorithm,"estimation with model formula:\n"))
    print(object$call$treatment_formula)
  }
  if (object$primary_estimator_name %in% c("outcome_modeling","doubly_robust")) {
    cat(paste0("\nOutcome model was ",object$arguments$outcome_algorithm," estimation ",
              ifelse(object$arguments$outcome_algorithm == "ranger","(estimated separately for each treatment condition) ",""),
              "with model formula:\n"))
    print(object$call$outcome_formula)
  }
  cat(paste0("\nFactual estimates are means within and disparities across ",object$arguments$category_name,".\n"))
  cat(paste0("Counterfactual estimates are under an intervention to set ",treatment_name,
             ifelse(all(object$arguments$counterfactual_assignments == 1), " to 1.\n",
                    ifelse(all(object$arguments$counterfactual_assignments == 0)," to 0.\n",
                           ifelse(length(object$arguments$counterfactual_assignments) == 1, paste("to",object$arguments$counterfactual_assignments),
                                  " by a rule for counterfactual_assignments that varies across individuals.\n"))),
             ifelse(object$arguments$se,paste("Standard errors are calculated from",object$arguments$bootstrap_samples,"bootstrap samples.\n"),"")))
  print(object)
  cat("\nType plot(name_of_this_object) to visualize results.\n")
}

#' Coerce to a Data Frame
#' @description This function converts a \code{gapclosing} object into a data frame. The gapclosing class contains results within a named list, thus simplifying things for manual user interaction with the results. In some programming settings (e.g. a bootstrap), it is easier to work with a rectangular data frame of results. This function produces that data frame.
#' @param x Object of class \code{gapclosing}, produced by a call to \code{gapclosing()}.
#' @param ... Additional arguments to be passed to or from methods.
#' @return A data frame containing estimates.
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @importFrom magrittr %>%
#' @importFrom foreach %do%
#' @export
#'
#' @examples
#' # Simulate example data
#' simulated_data <- generate_simulated_data(n = 100)
#'
#' # Fit by outcome modeling
#' estimate <- gapclosing(
#'   data = simulated_data,
#'   outcome_formula = formula(outcome ~ treatment * category + confounder),
#'   treatment_name = "treatment",
#'   category_name = "category",
#'   counterfactual_assignments = 1
#' )
#' summary(estimate)
#'
#' # Convert to a data frame
#' estimate.df <- as.data.frame(estimate)

as.data.frame.gapclosing <- function(x, ...) {

  # Initialize some objects for non-standard evaluation
  estimator <- stringsAsFactors <- change_type <- change_formula <- NULL

  primary_estimator_name <- x$primary_estimator_name

  factual_results_df <- x$factual_means %>%
    dplyr::mutate(estimand = "factual_means") %>%
    # Coerce category to character to match the disparities format
    dplyr::mutate(dplyr::across(x$arguments$category_name, function(value) as.character(value))) %>%
    dplyr::bind_rows(x$factual_disparities %>%
                       dplyr::mutate(estimand = "factual_disparities")) %>%
    dplyr::mutate(estimator = "mean")
  counterfactual_results_df <- do.call(rbind, lapply(names(x$all_estimators), function(estimator_name) {
    this_estimator_results_df <- x$all_estimators[[estimator_name]]$counterfactual_means %>%
      dplyr::mutate(estimand = "counterfactual_means") %>%
      # Coerce category to character to match the disparities format
      dplyr::mutate(dplyr::across(x$arguments$category_name, function(value) as.character(value))) %>%
      dplyr::bind_rows(x$all_estimators[[estimator_name]]$counterfactual_disparities %>%
                         dplyr::mutate(estimand = "counterfactual_disparities")) %>%
      dplyr::bind_rows(x$all_estimators[[estimator_name]]$change_means %>%
                         dplyr::mutate(estimand = paste("change_means",change_type,change_formula, sep = "__")) %>%
                         dplyr::select(-change_type,-change_formula) %>%
                         # Coerce category to character to match the disparities format
                         dplyr::mutate(dplyr::across(x$arguments$category_name, function(value) as.character(value)))) %>%
      dplyr::bind_rows(x$all_estimators[[estimator_name]]$change_disparities %>%
                         dplyr::mutate(estimand = paste("change_disparities",change_type,change_formula, sep = "__")) %>%
                         dplyr::select(-change_type,-change_formula)) %>%
      dplyr::mutate(estimator = estimator_name)
  }))
  data.frame(factual_results_df %>%
               dplyr::bind_rows(counterfactual_results_df) %>%
               dplyr::mutate(primary = estimator %in% c("mean",primary_estimator_name)),
             ...)
}

#' Plot function for gapclosing objects
#' @description Produces summary plots for a gapclosing object.
#' @param x An object of class \code{gapclosing}, which results from a call to the function \code{gapclosing}
#' @param return_plots Logical, defaults to FALSE. If TRUE, returns a list of the 4 plots without printing. Defaults to FALSE, in which case the console will interactively ask the user to hit "return" to proceed through printouts of the four plots, with no plots returned.
#' @param arranged Logical, defaults to FALSE. If TRUE, returns a list of the 4 plots arranged in a 2x2 table. Useful to visualize all four in one screen.
#' @param ... Other arguments to \code{plot} commands
#' @return If \code{return_plots = TRUE}, returns a list of \code{ggplot2} objects. If \code{return_plots = FALSE} (the default), then nothing is returned and output is printed.
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @importFrom foreach %do%
#' @importFrom magrittr %>%
#' @export

plot.gapclosing <- function(x, return_plots = FALSE, arranged = FALSE, ...) {

  # Prepare for non-standard evaluation
  setting <- gapclosing.category <- estimate <- ci.min <- ci.max <- NULL

  mean_outcomes_data <- x$factual_means %>%
    dplyr::mutate(setting = "Factual") %>%
    dplyr::bind_rows(x$counterfactual_means %>%
                       dplyr::mutate(setting = "Counterfactual")) %>%
    dplyr::mutate(setting = forcats::fct_rev(setting))

  mean_outcomes <- mean_outcomes_data %>%
    dplyr::rename_with(function(this_name) dplyr::case_when(this_name == x$arguments$category_name ~ "gapclosing.category",
                                                            T ~ this_name)) %>%
    ggplot2::ggplot(ggplot2::aes(x = gapclosing.category, y = estimate,
                                 color = setting, shape = setting)) +
    ggplot2::geom_point(position = ggplot2::position_dodge(width = .2)) +
    ggplot2::theme_bw() +
    ggplot2::ylab("Mean Outcome") +
    ggplot2::xlab("Category") +
    ggplot2::theme(legend.title = ggplot2::element_blank())

  # If there was a standard error, add the CI to the plot
  if (x$arguments$se) {
    mean_outcomes <- mean_outcomes +
      ggplot2::geom_errorbar(ggplot2::aes(ymin = ci.min, ymax = ci.max),
                             width = .1,
                             position = ggplot2::position_dodge(width = .2))
  }


  unique_categories <- unique(x$factual_means[[x$arguments$category_name]])
  num_unique_categories <- length(unique_categories)
  category_pairs <- matrix(NA, nrow = 0, ncol = 2, dimnames = list(NULL, c("category_A","category_B")))
  for (i in 1:(num_unique_categories - 1)) {
    category_pairs <- rbind(category_pairs,
                            cbind(unique_categories[i],
                                  unique_categories[(i+1):num_unique_categories]))
  }

  pairwise_plots <- lapply(1:nrow(category_pairs), function(i) {
    disparityplot(x, category_pairs[i,1], category_pairs[i,2])
  })
  names(pairwise_plots) <- apply(category_pairs,1, function(value) paste0("gap_",value[1],"_",value[2]))

  if (!return_plots) {
    # Print the plots to the viewer
    print(mean_outcomes)
    for (i in 1:length(pairwise_plots)) {
      readline(prompt = "Press return to see the next plot")
      print(pairwise_plots[[i]])
    }
    print("If you would prefer to save these plots, use the argument return_plots = TRUE")
  } else {
    # If requested, return the plots as a list
    print("Returning a list containing a series of plots")
    return(c(mean_outcomes = list(mean_outcomes), pairwise_plots))
  }
}

#' Convert Back to Canonical List Output
#' @description If the user has used \code{as.data.frame(x)} to convert a \code{gapclosing} object to a data frame of estimates, this function will invert back to the original list format. This function does not fully reinstate the original gapclosing object because some elements are lost when \code{as.data.frame()} is called. This function is most useful as a check on \code{as.data.frame()} and as a helper in settings like bootstrapping where a data frame is easier to work with but we want to return to the original format before returning an object to the user.
#' @param x A data frame produced by \code{as.data.frame(x)} applied to an object \code{x} of class \code{gapclosing}.
#' @return A list containing a subset of the elements in a \code{gapclosing} object.
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @importFrom magrittr %>%
#' @export
#'
#' @examples
#' # Simulate example data
#' simulated_data <- generate_simulated_data(n = 100)
#'
#' # Fit by outcome modeling
#' estimate <- gapclosing(
#'   data = simulated_data,
#'   outcome_formula = formula(outcome ~ treatment * category + confounder),
#'   treatment_name = "treatment",
#'   category_name = "category",
#'   counterfactual_assignments = 1
#' )
#' summary(estimate)
#'
#' # Convert to a data frame
#' estimate.df <- as.data.frame(estimate)
#' # Convert back to a list
#' estimate.df <- df_to_gapclosing_list(estimate.df)

df_to_gapclosing_list <- function(x) {

  # Initialize some objects for non-standard evaluation
  estimator <- estimand <- primary <- NULL

  # The gapclosing object uses tibbles, so use a tibble
  x <- tidyr::as_tibble(x)

  factual <- list(factual_means = x %>%
                    dplyr::filter(estimand == "factual_means") %>%
                    dplyr::select(-estimand,-estimator,-primary),
                  factual_disparities = x %>%
                    dplyr::filter(estimand == "factual_disparities") %>%
                    dplyr::select(-estimand,-estimator,-primary))
  estimator_names <- unique(x$estimator)
  estimator_names <- estimator_names[estimator_names != "mean"]
  all_estimators <- lapply(estimator_names, function(estimator_case) {
    list(counterfactual_means = x %>%
           dplyr::filter(estimand == "counterfactual_means" & estimator == estimator_case) %>%
           dplyr::select(-estimand,-estimator,-primary),
         counterfactual_disparities = x %>%
           dplyr::filter(estimand == "counterfactual_disparities" & estimator == estimator_case) %>%
           dplyr::select(-estimand,-estimator,-primary),
         change_means = x %>%
           dplyr::filter(grepl("change_means",estimand) & estimator == estimator_case) %>%
           tidyr::separate(estimand, into = c("estimand", "change_type", "change_formula"), sep = "__") %>%
           dplyr::select(-estimand,-estimator,-primary),
         change_disparities = x %>%
           dplyr::filter(grepl("change_disparities",estimand) & estimator == estimator_case) %>%
           tidyr::separate(estimand, into = c("estimand", "change_type", "change_formula"), sep = "__") %>%
           dplyr::select(-estimand,-estimator,-primary))
  })

  names(all_estimators) <- estimator_names
  primary_estimator_name <- unique(x$estimator[x$primary & x$estimator != "mean"])
  return(c(factual,
           all_estimators[[primary_estimator_name]],
           all_estimators = list(all_estimators)))
}

#' Cross-fitting gap closing estimator
#' @description This is an internal function typically called from other functions rather than by the user. It creates cross-validation folds and repeatedly calls split_sample_estimator to conduct cross-fitting.
#' @param data Data frame containing the observed data
#' @param counterfactual_assignments Numeric scalar or vector of length nrow(data), each element of which is on the \[0,1\] interval. If a scalar, the counterfactual probability by which all units are assigned to treatment condition 1. If a vector, each element i corresponds to the counterfactual probability by which each unit i is assigned to treatment condition 1.
#' @param outcome_formula Model formula the outcome. Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph). If \code{outcome_algorithm} = "ranger", then the outcome model will be fit separately on the treatment and control groups. Otherwise, the user must specify all interactions in the formula.
#' @param treatment_formula Treatment formula, in the style formula(treatment ~ covariates). Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph).
#' @param category_name Character name of the variable indicating the categories over which the gap is defined. Must be the name of a column in \code{data}.
#' @param outcome_name Character name of the outcome variable. Only required when there is no outcome_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.
#' @param treatment_name Character name of the treatment variable. Only required when there is no treatment_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.
#' @param treatment_algorithm Character name of the algorithm for the treatment model. One of "glm", "ridge", "gam", or "ranger". Defaults to "glm", which is a logit model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}). If "ranger", this function avoids propensity scores equal to 0 or 1 by bottom- and top-coding predicted values at .001 and .999.
#' @param outcome_algorithm Character name of the algorithm for the outcome model. One of "lm", "ridge", "gam", or "ranger". Defaults to "lm", which is an OLS model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}).
#' @param weight_name Character name of a sampling weight variable, if any, which captures the inverse probability of inclusion in the sample. The default assumes a simple random sample (all weights equal).
#' @param n_folds Only used if \code{method} = "cross_fit" and if \code{folds} is not provided. Integer scalar containing number of cross-validation folds. The function will assign observations to folds systematically: sort the data by the variable named \code{category_name}, then by the treatment variable, then at random. On this sorted dataset, folds are assigned systematically by repeated \code{1:n_folds}. To be used if the user does not provide \code{folds}. Defaults to 2.
#' @param folds_name Only used if \code{method} = "cross_fit". Character string indicating a column of \code{data} containing fold identifiers. This may be preferable to \code{n_folds} if the researcher has a reason to assign the folds in these data by some other process, perhaps due to particulars of how these data were generated. If null (the default), folds are assigned as stated in \code{n_folds}.
#' @return A list with four elements.
#' \itemize{
#' \code{counterfactual_means} A tibble with a counterfactual mean estimate for each category
#' \code{counterfactual_means} A tibble with a counterfactual disparity estimate for each pair of categories
#' \code{treatment_model} Object containing the fitted treatment model
#' \code{outcome_model} Object containing the fitted outcome model
#' }
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @importFrom foreach %do%
#' @importFrom magrittr %>%
#' @export

cross_fit_estimator <- function(
  data,
  counterfactual_assignments,
  outcome_formula,
  treatment_formula,
  category_name,
  outcome_name,
  treatment_name,
  treatment_algorithm = "glm",
  outcome_algorithm = "lm",
  weight_name = NULL,
  n_folds = 2,
  folds_name = NULL
) {
  # Initialize values to silence errors from non-standard evaluations
  gapclosing.folds <- f <- estimate <- NULL

  # If explicit fold assignments were not given, assign folds here
  if (!is.null(folds_name)) {
    n_folds <- length(unique(data[[folds_name]]))
    data$gapclosing.folds <- data[[folds_name]]
  }
  if (is.null(folds_name)) {
    # Use a cross-validation design to balance folds across categories and treatments
    # Sort data systematically by category and treatment, then randomly within those
    data_sort <- order(data[[category_name]],data[[treatment_name]],stats::runif(nrow(data)))
    data <- data[data_sort,]
    data$gapclosing.folds <- rep(1:n_folds, ceiling(nrow(data) / n_folds))[1:nrow(data)]
  }
  # Initialize a list to hold the fold estimates
  fold_estimates <- foreach::foreach(f = unique(data$gapclosing.folds)) %do% {
    point_estimator(
      data_learn = data %>% dplyr::filter(gapclosing.folds != f),
      data_estimate = data %>% dplyr::filter(gapclosing.folds == f),
      outcome_formula = outcome_formula,
      treatment_formula = treatment_formula,
      outcome_name = outcome_name,
      treatment_name = treatment_name,
      category_name = category_name,
      counterfactual_assignments = ifelse(length(counterfactual_assignments == 1),
                                          counterfactual_assignments,
                                          counterfactual_assignments[data$gapclosing.folds == f]),
      weight_name = weight_name,
      treatment_algorithm = treatment_algorithm,
      outcome_algorithm = outcome_algorithm
    )
  }
  names(fold_estimates) <- paste0("fold_",unique(data$gapclosing.folds),"_left_out")

  # Pool the counterfactual_means and counterfactual_disparities results across cross-fits
  pooled <- lapply(
    c(counterfactual_means = "counterfactual_means", counterfactual_disparities = "counterfactual_disparities"),
    function(element) {
      do.call(rbind,lapply(fold_estimates, function(fold_result) fold_result[[element]])) %>%
        dplyr::group_by(dplyr::across(tidyselect::all_of(c(category_name,"method")))) %>%
        dplyr::summarize(estimate = mean(estimate),
                         .groups = "drop")
    }
  )

  object_to_return <- list(counterfactual_means = pooled$counterfactual_means,
                           counterfactual_disparities = pooled$counterfactual_disparities,
                           treatment_model = lapply(fold_estimates, function(x) x$treatment_model),
                           outcome_model = lapply(fold_estimates, function(x) x$outcome_model))

  return(object_to_return)
}

#' Print function for gapclosing objects
#' @description Prints the same output as generated by \code{summary}
#' @param x An object of class \code{gapclosing}, which results from a call to the function \code{gapclosing}
#' @param ... Other arguments to \code{print} commands
#' @param digits Argument passed to print.data.frame
#' @param quote Argument passed to print.data.frame
#' @param right Argument passed to print.data.frame
#' @param row.names Argument passed to print.data.frame
#' @param max Argument passed to print.data.frame
#' @return Prints a summary of the estimates.
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @importFrom magrittr %>%
#' @export

print.gapclosing <- function(
  x,
  ...,
  digits = 2,
  quote = FALSE,
  right = FALSE,
  row.names = FALSE,
  max = NULL
) {

  # Prepare for non-standard evaluation
  change_type <- change_formula <- NULL

  cat("\nFactual mean outcomes:\n")
  print(x$factual_means,
        digits = digits,
        quote = quote,
        right = right,
        row.names = row.names,
        max = max, scientific = FALSE)
  cat("\nCounterfactual mean outcomes (post-intervention means):\n")
  print(x$counterfactual_means,
        digits = digits,
        quote = quote,
        right = right,
        row.names = row.names,
        max = max, scientific = FALSE)
  cat("\nFactual disparities:\n")
  print(x$factual_disparities,
        digits = digits,
        quote = quote,
        right = right,
        row.names = row.names,
        max = max, scientific = FALSE)
  cat("\nCounterfactual disparities (gap-closing estimands):\n")
  print(x$counterfactual_disparities,
        digits = digits,
        quote = quote,
        right = right,
        row.names = row.names,
        max = max, scientific = FALSE)
  cat("\nAdditive gap closed: Counterfactual - Factual\n")
  print(x$change_disparities %>%
          dplyr::filter(change_type == "additive") %>%
          dplyr::select(-change_type, -change_formula),
        digits = digits,
        quote = quote,
        right = right,
        row.names = row.names,
        max = max, scientific = FALSE)
  cat("\nProportional gap closed: (Counterfactual - Factual) / Factual\n")
  print(x$change_disparities %>%
          dplyr::filter(change_type == "proportional") %>%
          dplyr::select(-change_type, -change_formula),
        digits = digits,
        quote = quote,
        right = right,
        row.names = row.names,
        max = max, scientific = FALSE)
}

#' Ridge regression estimator
#' @description Not typically called by the user directly; called indirectly via other functions. Uses \code{glmnet} to fit a ridge regression with penalty chosen by cross-validation. Returns fitted values for the data in \code{to_predict}.
#' @param data Data frame containing the observed data
#' @param model_formula A model formula object for the ridge regression to be fitted
#' @param to_predict Data frame containing observations for which predictions are to be made. If NULL, defaults to the same as \code{data}.
#' @return A list containing a \code{glmnet} model object \code{fit} and a vector \code{fitted} of fitted values for observations in \code{to_predict}.
#' @references Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
#' @references Friedman J, Hastie T, Tibshirani R (2010). "Regularization Paths for Generalized Linear Models via Coordinate Descent." Journal of Statistical Software, 33(1), 1–22. {https://www.jstatsoft.org/htaccess.php?volume=33&type=i&issue=01}.
#' @export

fit_ridge <- function(data, model_formula, to_predict) {
  if (is.null(data$gapclosing.weight)) {
    stop("ERROR: fit_ridge requires that data contains a variable named weight. The package should have created this internally in a higher function.")
  }
  outcome <- data[[as.character(model_formula)[2]]]
  X <- stats::model.matrix(model_formula, data = data)
  X_predict <- stats::model.matrix(model_formula, data = to_predict, alpha = 0)
  if (!all(colnames(X) == colnames(X_predict))) {
    stop("Error: Within ridge estimation, the model matrices for learning and prediction have different column names")
  }
  fit <- glmnet::cv.glmnet(x = X, y = outcome,
                           family = ifelse(all(outcome %in% 0:1), "binomial", "gaussian"),
                           weights = data[["weight"]])
  fitted <- stats::predict(fit, newx = X_predict, type = "response", s = fit$lambda.min)
  return(list(fitted = fitted[,1], fit = fit))
}

---
title: "gapclosing: An R package to estimate gap closing estimands"
output: rmarkdown::html_vignette
header-includes: 
  - \usepackage{tikz}
  - \usepackage{pgfplots}
vignette: >
  %\VignetteIndexEntry{gapclosing}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: bibliography.bib
---

<style type="text/css">
code.r{
  font-size: 12px;
}
pre {
  font-size: 12px
}
</style>

```{r setup, include=FALSE}
knitr::opts_chunk$set(dpi = 300, fig.width = 6.5, fig.height = 3, out.width = "650px")
```

```{r, include = FALSE}
t0 <- Sys.time()
```

Gaps across social categories like race, class, and gender are important to understand. We would like to know whether there is anything we can do to close these gaps. What if we intervened to reduce incarceration or increase access to education? Would those interventions close gaps across categories of race, class, or gender?

These types of questions are at the core of a growing literature in epidemiology addresses these questions with techniques for causal decomposition analysis (@vanderweele2014, @jackson2018, @jackson2020). The paper which accompanies this software package introduces these methods to a broader set of social scientists.

>Lundberg, Ian. Forthcoming. "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." _Sociological Methods and Research_. Preprint available at [https://doi.org/10.31235/osf.io/gx4y3](https://doi.org/10.31235/osf.io/gx4y3).

This package provides software to support inquiry into gap-closing estimands. A guiding principle is to partition the tasks the human user must do from the tasks that the package can automate.

The human user must carry out these tasks:

* Define the intervention
* Make causal assumptions for identification
* Specify a treatment model and/or an outcome model

Given those inputs, the package will automatically:

* Estimate models (GLM, GAM, ridge regression, or random forest)
* Convert models to doubly-robust estimates of gap-closing estimands, using sample splitting to improve convergence
* Estimate standard errors by bootstrapping
* Visualize the result

The structure of the vignette is as follows.

1. Data structure
2. Quick overview: Functionality
3. Detailed procedures: Customize your call
    * Define the intervention
    * Make causal assumptions for identification
    * Specify a treatment model and/or an outcome model
4. Estimate by machine learning: Two examples
5. Estimate with stochastic assignment rules
6. Conclusion

# 1. Data structure

In a data frame `data`, we have a gap-defining category such as race, gender, or class. We have a binary treatment variable that could have counterfactually been different for any individual. We want to know the degree to which an intervention to change the treatment would close gaps across the categories.

> **These boxes will present an example.**
> 
> *Example.* Suppose we have the following data.
> 
>* $X$ (`category`): Category of interest, taking values {A, B, C}
>* $T$ (`treatment`): Binary treatment variable, taking values 0 and 1
>* $L$ (`confounder`): A continuous confounding variable, Uniform(-1,1)
>* $Y$ (`outcome`): A continuous outcome variable, conditionally normal

```{r, message = FALSE, warning = FALSE}
set.seed(08544)
library(gapclosing)
library(dplyr)
library(ggplot2)
```
```{r}
simulated_data <- generate_simulated_data(n = 1000)
head(simulated_data)
```

# 2. Quick overview: Functionality

The `gapclosing()` function estimates gaps across categories and the degree to which they would close under the specified `counterfactual_assignments` of the treatment.

```{r}
estimate <- gapclosing(
  data = simulated_data,
  counterfactual_assignments = 1,
  outcome_formula = formula(outcome ~ confounder + category*treatment),
  treatment_formula = formula(treatment ~ confounder + category),
  category_name = "category",
  se = TRUE,
  # You can process the bootstrap in parallel with as many cores as available
  parallel_cores = 2
)
```
By default, this function will do the following:

* Fit logistic regression to predict treatment assignment
* Fit OLS regression to predict outcomes
* Combine the two in a doubly-robust estimator estimated on a single sample
* Return a `gapclosing` object which supports `summary`, `print`, and `plot` functions.

In this example, the `plot(estimate)` function produces the following visualization. The factual outcomes are unequal across categories, but the counterfactual outcomes are roughly equal. In this simulated setting, the intervention almost entirely closes the gaps across the categories.

```{r, include = FALSE}
plots <- plot(estimate, return_plots = TRUE)
```
```{r, echo = FALSE, fig.width = 5, fig.height = 3.5, out.width = "650px", fig.cap = "Figure 1 produced by plot() function"}
print(plots[[1]] +
        ggtitle("First result of a call to plot()"))
```

The `disparityplot()` function lets us zoom in on the factual and counterfactual disparity between two categories, of interest. In this case, we see that the intervention lifts outcomes in category A to be more comparable to category B. A `disparityplot` is a `ggplot2` object and can be customized by passing additional layers.

```{r, echo = FALSE, fig.width = 5, fig.height = 3.5, out.width = "650px", fig.cap = "Figure 1 produced by plot() function"}
disparityplot(estimate, category_A = "A", category_B = "B") +
  ggtitle("A disparityplot()")
```

The `summary` function will print estimates, standard errors, and confidence intervals for all of these results.

```{r, include = FALSE}
old <- options()
options(width = 300)
```
```{r}
summary(estimate)
```
```{r, include = FALSE}
options(old)
```

# 3. Detailed procedures: Customize your call

This section provides a more detailed overview of the use of `gapclosing()`. It is structured from the perspective of the three key tasks for the researcher: define the intervention, make causal assumptions for identification, and specify a treatment and/or outcome model. Along the way, this section introduces many of the possible arguments in a `gapclosing()` call.

## Step 1. Define the intervention

 To answer a gap-closing question, we first need to define what that intervention would be. To what treatment value would units be counterfactually assigned? There are several options.

* **Option 1.** Set `counterfactual_assignments = 0` or `counterfactual_assignments = 1`. In this case, we are studying the disparity we would expect if a random person of each category were assigned control or if assigned treatment (respectively).
* **Option 2.** Set `counterfactual_assignments` = $\pi$ for some $\pi$ between 0 and 1. In this case, we are studying the disparity we would expect if a random person of each category were assigned to treatment with probability $\pi$ and to control with probability $1 - \pi$.
* **Option 3.** Assign treatments by a probability $\pi_i$ that may differ for each person $i$. In this case, we create a vector $\vec\pi$ of length $n$ and pass that in the `counterfactual_assignments` argument.

> *Example.* We are interested in the disparity across populations defined by `category` that would persist under counterfactual assignment to set `treatment` to the value 1.
>`counterfactual_assignments = 1`

## Step 2. Make causal assumptions for identification

The package does help you with this step. Gap-closing estimands involve unobserved potential outcomes. Because they are unobserved, the data cannot tell us which variables are needed for estimation. Instead, that is a conceptual choice to be carried out with tools like Directed Acyclic Graphs (DAGs). See the accompanying @lundberg2021 paper for more on identification.

>*Example.* Assume that the set of variables $\{X,L\}$ is a sufficient conditioning set to identify the gap-closing estimand. Formally, this requires us to assume that within each stratum of $X$ and $L$ the expected value of the potential outcome $Y(1)$ is the same as the expected value among units who factually have $T = 1$ within those strata.
$$\mathbb{E}(Y(1)\mid X, L) = \mathbb{E}(Y\mid X, L, T = 1)$$
DAGs are a good way to reason about this assumption: in this example, conditioning (depicted by boxes) on `category` and `confounder` is sufficient to identify the causal effect of `treatment` (blue edge in the DAG), because doing so blocks all backdoor paths between the treatment and the outcome. Notably, the gap-closing estimand makes no claims about the causal effect of `category` since the counterfactual is defined over `treatment` only.

```{r, echo = FALSE, fig.height = 2}
for_DAG <- data.frame(label = c("Category","Confounder","Treatment","Outcome","Unobserved"),
                      x = c(1,2,3,4,1),
                      y = c(1,1,1,1,0))
for_DAG %>%
  ggplot(aes(x = x, y = y, label = label)) +
  # Edges coming out of category
  annotate(geom = "segment",
         x = 1, y = 1, xend = 1.55, yend = 1,
         arrow = arrow(length = unit(.1,"in"))) +
  annotate(geom = "curve",
           x = 1, y = 1, xend = c(2.9,3.9), yend = c(1.2,1.2),
           arrow = arrow(length = unit(.1,"in")),
           curvature = -.3) +
  # Edges coming out of confounder
  annotate(geom = "segment",
           x = 2, y = 1, xend = 2.65, yend = 1,
           arrow = arrow(length = unit(.1,"in"))) +
  annotate(geom = "curve",
           x = 2, y = 1, xend = 3.75, yend = .82,
           arrow = arrow(length = unit(.1,"in")),
           curvature = .25) +
  # Edges coming out of treatment
  annotate(geom = "segment",
           x = 3, y = 1, xend = 3.65, yend = 1,
           color = "blue", size = 1.05,
           arrow = arrow(length = unit(.13,"in"))) +
  # Edges coming out of unobserved
  annotate(geom = "segment",
           x = 1, y = 0, xend = c(1,1.9), yend = .8,
           arrow = arrow(length = unit(.1,"in"))) +
  annotate(geom = "curve",
           x = 1, y = 0, xend = c(3.9), yend = .8,
           arrow = arrow(length = unit(.1,"in")),
           curvature = .25) +
  geom_label(color = "white") +
  geom_label(data = for_DAG %>% filter(label %in% c("Category","Confounder"))) +
  geom_text(data = for_DAG %>% filter(label %in% c("Unobserved","Treatment","Outcome"))) +
  ylim(c(-.3,2)) +
  xlim(c(0,5)) +
  theme_void()
```

Once we select a sufficient conditioning set, those predictors will appear in both the treatment and/or the outcome model used for estimation.

## Step 3. Specify a treatment model and/or an outcome model

We need to estimate one or both of (1) the probability of treatment given confounders and (2) the conditional mean of the outcome given treatment and confounders. We do that by providing one or both of the following.

* Provide a `treatment_formula` with the binary treatment variable on the left side and all confounders on the right side. This formula will be used in `treatment_algorithm` (see next step) to estimate the probability of treatment given confounders for each unit. Then, the sample-average outcome under counterfactual assignment to any given treatment can be estimated through inverse probability weighting among units who factually received that treatment.

>*Example.*  
>`treatment_formula = formula(outcome ~ confounder + category)`

* Provide an `outcome_formula` with the outcome variable on the left side and the treatment and all confounders on the right side. This formula will be used in `outcome_algorithm` (see next step) to estimate the conditional mean of the outcome given treatment and confounders. Then, it can be used to predict the potential outcome each unit would expect under any treatment of interest, given confounding variables.

>*Example.*  
>`outcome_formula = formula(outcome ~ confounder + category*treatment)`

Whether `treatment_formula` or `outcome_formula` is left `NULL` will determine the estimation procedure.

* If only `treatment_formula` is provided, then estimation will be by inverse probability of treatment weighting.
* If only `outcome_formula` is provided, then estimation will be by prediction of unobserved potential outcomes (the $g$-formula, see @hernan2021).
* If both `treatment_formula` and `outcome_formula` are provided, then the primary estimate will be doubly robust, with separate treatment and outcome modeling estimates accessible through the returned object.

### Choose an estimation algorithm

The `treatment_formula` and `outcome_formula` are handed to `treatment_algorithm` and `outcome_algorithm`, which can take the following values.

* `glm` (for `treatment_algorithm` only): A logistic regression model estimated by the `glm` function. If there are important interactions among treatment, category, and covariates, these should be included explicitly.
* `lm` (for `outcome_algorithm` only): An OLS regression model estimated by the `lm` function.  If there are important interactions among treatment, category, and covariates, these should be included explicitly.
* `ridge`: A ridge (i.e. L<sub>2</sub>-penalized) logistic regression model (for `treatment_algorithm`) or linear regression model (for `outcome_algorithm`) estimated by the `glmnet` function in the `glmnet` package [@glmnet], with `alpha = 0` to indicate the ridge penalty. Elastic net and lasso regression are not supported because those approaches could regularize some coefficients to exactly zero, and if you have chosen the needed confounders (step 2) then you would not want their coefficients to be regularized to zero. The penalty term is chosen by `cv.glmnet` and is set to the value `lambda.min` (see `glmnet` documentation).  If there are important interactions among treatment, category, and covariates, these should be included explicitly.
* `gam`: A Generalized Additive Model (logistic if used as `treatment_algorithm`, linear if used as `outcome_algorithm`) estimated by the `gam` function in the `mgcv` package [@mgcv]. The model formula should be specified as in the `mgcv` documentation and may include smooth terms `s()` for continuous covariates.  If there are important interactions among treatment, category, and covariates, these should be included explicitly.
* `ranger`: A random forest estimated by the `ranger` function in the `ranger` package [@ranger]. If used as `treatment_algorithm`, one forest will be fit and predicted treatment probabilities will be truncated to the [.001,.999] range to avoid extreme inverse probability of treatment weights. If used as `outcome_algorithm`, the forest will be estimated separately on treated and control units; the treatment variable does not need to be included in `outcome_formula` in this case.

>*Example.*  
> `treatment_algorithm = "glm"`  
> `outcome_algorithm = "lm"`  

If the data are a sample from a population selected with unequal probabilities, you can also use the `weight_name` option to pass estimation functions the name of the sampling weight (a variable in `data` proportional to the inverse probability of sample inclusion). If omitted, a simple random sample is assumed.

### Why doubly robust? A side note

Doubly-robust estimation yields advantages that can be conceptualized in two ways.

* From the perspective of parametric models, if either the treatment or the outcome model is correct, then the estimator is consistent [@bang2005]. See @glynn2010 for an accessible introduction.  
* From the perspective of machine learning, double robustness adjusts for the fact that outcome modeling alone is optimized for the wrong prediction task. An outcome model would be optimized to predict where we observed data, but our actual task is to predict over a predictor distribution _different_ from that which was observed (because the treatment has been changed). This is an opportunity to improve the predictions. By building a model of treatment, we can reweight the residuals of the outcome model to estimate the average prediction error over the space where we want to make predictions. Subtracting off this bias can improve the outcome modeling estimator. This pivot is at the core of moves toward targeted learning [@vanderlaan2011] and double machine learning [@chernozhukov2018], building on a long line of research in efficient estimation [@robins1995;@hahn1998].

Although double robustness has strong mathematical properties, in any given application with a finite sample it is possible that treatment or outcome modeling could outperform doubly-robust estimation. Therefore, the package supports all three approaches.

### Why sample splitting? Another side note

Taking the bias-correction view of double robustness above, it is clear that sample splitting affords a further opportunity for improvement: if you learn an outcome model _and_ estimate its average bias on the same sample, you might get a poor estimate of the bias. For this reason, one should consider using one sample (which I call `data_learn`) to learn the prediction functions and another sample (which I call `data_estimate`) to estimate the bias and aggregate to an estimate of the estimand.

In particular, the option `sample_split = "cross_fit"` allows the user to specify that estimation should proceed by a cross-fitting procedure which is analogous to cross-validation.

1. Split the sample into folds $f = 1,\dots,$`n_folds` (default here is `n_folds = 2`)  
2. Use all folds except $f'$ to estimate the treatment and outcome models  
3. Aggregate to an estimate using the predictions in $f'$  
4. Average the estimate that results from (2) and (3) repeated `n_folds` times with each fold playing the role of $f'$ in turn  

This is the procedure that @chernozhukov2018 argue is critical to double machine learning for causal estimation, although this type of sample splitting is not new [@bickel1982].

If `sample_split = "cross_fit"`, the default is to conduct 2-fold cross-fitting, but this can be changed with the `n_folds` argument. The user can also specify their own vector `folds` of fold assignments of length `nrow(data)`, if there is something about the particular setting that would make a manual fold assignment preferable.

>*Example.* (this is the default and can be left implicit)  
> `sample_split = "one_sample"`  

### Produce standard errors

The package supports bootstrapped standard error estimation. The procedure with `bootstrap_method = "simple"` (the default) is valid when the data are a simple random sample from the target population. In this case, each bootstrap iteration conducts estimation on a resampled dataset selected with replacement with equal probabilities. The standard error is calculated as the standard deviation of the estimate across bootstrap samples, and confidence intervals are calculated by a normal approximation.

>*Example.* (line 2 is the default and can be left implicit)  
> `se = T`
> `bootstrap_samples = 1000`

In some settings, the sample size may be small and categories or treatments of interest may be rare. In these cases, it is possible for one or more simple bootstrap samples to contain zero cases in some (treatment $\times$ category) cell of interest. To avoid this problem, `bootstrap_method = "stratified"` conducts bootstrap resampling within blocks defined by (treatment $\times$ category). This procedure is valid if you assume that the data are selected at random from the population within these strata, so that across repeated samples from the true population the proportion in each stratum would remain the same.

Many samples are not simple random samples. In complex sample settings, users should implement their own standard error procedures to accurately capture sampling variation related to how their data were collected. The way the data were collected could motivate a resampling strategy to mimic the sources of variation in that sampling process, which the user can implement manually by calling `gapclosing` to calculate a point estimate on each resampled dataset with `se = FALSE`.

# 4. Estimate by machine learning: Two examples

Suppose you want to relax parametric functional form assumptions by plugging in a machine learning estimator. As the user, this simply involves changing the arguments to the `gapclosing()` function.

## Estimation with Generalized Additive Models (GAMs) via `mgcv`

Perhaps you are concerned about linearity assumptions: the continuous confounder, for instance, might actually have a nonlinear association with the outcome. We know the truth is linear in this simulated example, but in practice you would never know. You can address this concern by estimating with a GAM, using the `s()` operator from `mgcv` for smooth terms (see @mgcv).

```{r, warning = FALSE}
estimate_gam <- gapclosing(
  data = simulated_data,
  counterfactual_assignments = 1,
  outcome_formula = formula(outcome ~ s(confounder) + category*treatment),
  treatment_formula = formula(treatment ~ s(confounder) + category),
  category_name = "category",
  treatment_algorithm = "gam",
  outcome_algorithm = "gam",
  sample_split = "cross_fit"
  # Note: Standard errors with `se = TRUE` are supported.
  # They are omitted here only to speed vignette build time.
)
```

## Estimation with random forests via `ranger`

Perhaps you are concerned that the true treatment probability and expected outcome functions have many interactions among the predictors. You can set `treatment_algorithm` and `outcome_algorithm` to "ranger" to estimate via the `ranger` function in the `ranger` package [@ranger].

One aspect of the way `gapclosing()` operationalizes `ranger()` is unique out of all the estimation algorithm options. When you choose a random forest, it is because you believe there are many important interactions. Some of the most important interactions may be between the treatment and the other predictors. Therefore, `outcome_algorithm = ranger` enforces those interactions by estimating the outcome model separately for treated and control units. For this reason, when `outcome_algorithm = ranger` there is no need to include the treatment variable explicitly in the `outcome_formula`.

```{r}
estimate_ranger <- gapclosing(
  data = simulated_data,
  counterfactual_assignments = 1,
  outcome_formula = formula(outcome ~ confounder + category),
  treatment_formula = formula(treatment ~ confounder + category),
  category_name = "category",
  treatment_algorithm = "ranger",
  outcome_algorithm = "ranger",
  sample_split = "cross_fit"
  # Note: Standard errors with `se = TRUE` are supported.
  # They are omitted here only to speed vignette build time.
)
```

## Estimates from these three algorithms are roughly the same

In this simulation, the GLM models are correctly specified and there are no nonlinearities or interactions for the machine learning approaches to learn. In this case, the sample size is large enough that those approaches correctly learn the linear functional form, and all three estimation strategies yield similar estimates.

```{r, include = FALSE}
glm_plot <- plot(estimate, return_plots = T)[[1]]
gam_plot <- plot(estimate_gam, return_plots = T)[[1]]
ranger_plot <- plot(estimate_ranger, return_plots = T)[[1]]
```

```{r, echo = FALSE, figures-side, fig.show="hold", out.width="30%", message = FALSE, warning = FALSE}
glm_plot +
    ggtitle("GLM estimate") +
    ylim(c(-1.5,1.5)) +
  theme(legend.position = "bottom")
gam_plot +
    ggtitle("GAM estimate") +
    ylim(c(-1.5,1.5)) +
  theme(legend.position = "bottom")
ranger_plot +
  ggtitle("Random forest estimate") +
  ylim(c(-1.5,1.5)) +
  theme(legend.position = "bottom")
```

Note that confidence intervals for GAM and random forest can also be generated with `SE = TRUE`, which is turned off here only to speed vignette build time.

## A warning about complex estimators

The assumptions of a parametric model are always doubtful, leading to a common question of whether one should always use a more flexible machine learning approach like `ranger`. In a very large sample, a flexible learner would likely be the correct choice. In the sample sizes of social science settings, the amount of data may sometimes be insufficient for these algorithms to discover a complex functional form. When the parametric assumptions are approximately true, the parametric estimators may have better performance in small sample sizes. What counts as "small" and "large" is difficult to say outside of any specific setting.

# 5. Estimate with stochastic assignment rules

The examples above focused on estimation for a fixed treatment assignment: assign to treatment 1 with probability 1. But we might also want to know about the gap-closing estimand if we assigned people to treatment stochastically with some probability between 0 and 1. The `counterfactual_assignments` argument can handle this possibility.

For example, consider the gap-closing estimand if assigned to treatment 1 with each probability .75.
```{r}
estimate_stochastic <- gapclosing(
  data = simulated_data,
  counterfactual_assignments = .75,
  outcome_formula = formula(outcome ~ confounder + category*treatment),
  treatment_formula = formula(treatment ~ confounder + category),
  category_name = "category"
)
```
The disparity between categories A and B under that stochastic intervention (0.75 probability of treatment = 1) is estimated to be `r round((estimate_stochastic$counterfactual_disparities %>% filter(category == "B - A"))$estimate,2)`, whereas under the previous deterministic intervention to assign treatment to the value 1 the disparity would be `r round((estimate$counterfactual_disparities %>% filter(category == "B - A"))$estimate,2)`. This illustrates an important point: the gap-closing estimand can be different depending on the counterfactual assignment rule, as the figure below shows for counterfactuals in which treatment is assigned with probabilities ranging from 0 to 1.
```{r, echo = FALSE, fig.caption = "A conceptual figure for which the package does not provide a general-purpose function."}
counterfactual_assignments_values <- seq(0,1,.25)
many_stochastic_estimates <- rep(NA, length(counterfactual_assignments_values))
for (i in 1:length(counterfactual_assignments_values)) {
  estimate_case <- gapclosing(
    data = simulated_data,
    counterfactual_assignments = counterfactual_assignments_values[i],
    outcome_formula = formula(outcome ~ confounder + category*treatment),
    treatment_formula = formula(treatment ~ confounder + category),
    category_name = "category"
  )
  estimate_we_want <- estimate_case$change_disparities %>%
    filter(change_type == "proportional") %>%
    filter(category == "B - A")
  many_stochastic_estimates[i] <- estimate_we_want$estimate
}
data.frame(counterfactual_assignments = counterfactual_assignments_values,
           estimate = many_stochastic_estimates) %>%
  ggplot(aes(x = counterfactual_assignments,
             y = estimate)) +
  geom_hline(yintercept = c(0,1),
             color = "gray", linetype = "dashed") +
  geom_line(color = "gray") +
  geom_point(aes(color = factor(counterfactual_assignments == .75))) +
  xlab("Counterfactual Treatment Assignment Probability") +
  scale_y_continuous(name = "Percent of B - A Gap Closed",
                     labels = function(x) paste0(round(100*x),"%")) +
  annotate(geom = "text",
           x = 1, y = c(0,1),
           label = c("No change to disparity",
                     "Disparity completely eliminated"),
           color = "darkgray", size = 3, hjust = 1, vjust = 1.5) +
  annotate(geom = "text",
           x = .1, y = .3, 
           label = "Disparities eliminated under various\nstochastic\ninterventions",
           color = "gray", size = 3, hjust = 0, vjust = 1) +
  annotate(geom = "text",
           x = .75, y = .3,
           label = "Gap-Closing Estimand\nCalculated Above",
           color = "blue", size = 3, vjust = -.5) +
  scale_color_manual(values = c("gray","blue")) +
  theme_bw() +
  theme(panel.grid = element_blank(),
        legend.position = "none")
```

Your stochastic assignments can also be different for different people. For example, suppose we assign those in Category A to treatment 1 with probability .5, those in Category B to treatment with probability .4, and those in Category C to treatment with probability .3. In this case, `counterfactual_assignments` will be set to a vector of length `nrow(data)`.
```{r}
our_assignments <- case_when(simulated_data$category == "A" ~ .5,
                             simulated_data$category == "B" ~ .4, 
                             simulated_data$category == "C" ~ .3)
estimate_stochastic <- gapclosing(
  data = simulated_data,
  counterfactual_assignments = our_assignments,
  outcome_formula = formula(outcome ~ confounder + category*treatment),
  treatment_formula = formula(treatment ~ confounder + category),
  category_name = "category"
)
```
That intervention would close the B - A gap by `r paste0(round(100 * (estimate_stochastic$change_disparities %>% filter(category == "B - A" & change_type == "proportional"))$estimate),"%")`.

# 6. Conclusion

The `gapclosing` package is designed to support inquiry into gap closing estimands, thus promoting new understanding about interventions that can close gaps across social categories. The goal of the package is to automate technical tasks (sample splitting, aggregation to doubly robust estimates, visualization), thus freeing the researcher to devote more attention to scientific tasks like defining the intervention and making causal assumptions.

If you use this package and find a bug, it would be most helpful if you would create an issue on [GitHub](https://github.com/ilundberg/gapclosing). Suggestions for additional features are also welcome.

This vignette was compiled on `r Sys.time()`.

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gapclosing.R
\name{plot.gapclosing}
\alias{plot.gapclosing}
\title{Plot function for gapclosing objects}
\usage{
\method{plot}{gapclosing}(x, return_plots = FALSE, arranged = FALSE, ...)
}
\arguments{
\item{x}{An object of class \code{gapclosing}, which results from a call to the function \code{gapclosing}}

\item{return_plots}{Logical, defaults to FALSE. If TRUE, returns a list of the 4 plots without printing. Defaults to FALSE, in which case the console will interactively ask the user to hit "return" to proceed through printouts of the four plots, with no plots returned.}

\item{arranged}{Logical, defaults to FALSE. If TRUE, returns a list of the 4 plots arranged in a 2x2 table. Useful to visualize all four in one screen.}

\item{...}{Other arguments to \code{plot} commands}
}
\value{
If \code{return_plots = TRUE}, returns a list of \code{ggplot2} objects. If \code{return_plots = FALSE} (the default), then nothing is returned and output is printed.
}
\description{
Produces summary plots for a gapclosing object.
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pairwise_diff.R
\name{pairwise_diff}
\alias{pairwise_diff}
\title{Pairwise difference calculator}
\usage{
pairwise_diff(category_means_data, category_name)
}
\arguments{
\item{category_means_data}{Data frame containing two columns: the category-specific mean estimate (a column named \code{estimate}) and the category name (named as specified in \code{category_name})}

\item{category_name}{The name of the column containing the category identifier.}
}
\value{
A data frame with pairwise differences of \code{estimate} over pairs of categories.
}
\description{
A function to estimate the pairwise differences of estimates made for each category.
}
\examples{
sim_data <- data.frame(example_category = c("A","B","C"),
                       estimate = c(1,2,3))
pairwise_diff(sim_data, category_name = "example_category")
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.

Friedman J, Hastie T, Tibshirani R (2010). "Regularization Paths for Generalized Linear Models via Coordinate Descent." Journal of Statistical Software, 33(1), 1–22. {https://www.jstatsoft.org/htaccess.php?volume=33&type=i&issue=01}.

Wood S (2017). Generalized Additive Models: An Introduction with R, 2 edition. Chapman and Hall/CRC.

Wright MN, Ziegler A (2017). "ranger: A Fast Implementation of Random Forests for High Dimensional Data in C++ and R." Journal of Statistical Software, 77(1), 1–17. doi: 10.18637/jss.v077.i01.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.gapclosing.R
\name{print.gapclosing}
\alias{print.gapclosing}
\title{Print function for gapclosing objects}
\usage{
\method{print}{gapclosing}(
  x,
  ...,
  digits = 2,
  quote = FALSE,
  right = FALSE,
  row.names = FALSE,
  max = NULL
)
}
\arguments{
\item{x}{An object of class \code{gapclosing}, which results from a call to the function \code{gapclosing}}

\item{...}{Other arguments to \code{print} commands}

\item{digits}{Argument passed to print.data.frame}

\item{quote}{Argument passed to print.data.frame}

\item{right}{Argument passed to print.data.frame}

\item{row.names}{Argument passed to print.data.frame}

\item{max}{Argument passed to print.data.frame}
}
\value{
Prints a summary of the estimates.
}
\description{
Prints the same output as generated by \code{summary}
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cross_fit_estimator.R
\name{cross_fit_estimator}
\alias{cross_fit_estimator}
\title{Cross-fitting gap closing estimator}
\usage{
cross_fit_estimator(
  data,
  counterfactual_assignments,
  outcome_formula,
  treatment_formula,
  category_name,
  outcome_name,
  treatment_name,
  treatment_algorithm = "glm",
  outcome_algorithm = "lm",
  weight_name = NULL,
  n_folds = 2,
  folds_name = NULL
)
}
\arguments{
\item{data}{Data frame containing the observed data}

\item{counterfactual_assignments}{Numeric scalar or vector of length nrow(data), each element of which is on the [0,1] interval. If a scalar, the counterfactual probability by which all units are assigned to treatment condition 1. If a vector, each element i corresponds to the counterfactual probability by which each unit i is assigned to treatment condition 1.}

\item{outcome_formula}{Model formula the outcome. Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph). If \code{outcome_algorithm} = "ranger", then the outcome model will be fit separately on the treatment and control groups. Otherwise, the user must specify all interactions in the formula.}

\item{treatment_formula}{Treatment formula, in the style formula(treatment ~ covariates). Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph).}

\item{category_name}{Character name of the variable indicating the categories over which the gap is defined. Must be the name of a column in \code{data}.}

\item{outcome_name}{Character name of the outcome variable. Only required when there is no outcome_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.}

\item{treatment_name}{Character name of the treatment variable. Only required when there is no treatment_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.}

\item{treatment_algorithm}{Character name of the algorithm for the treatment model. One of "glm", "ridge", "gam", or "ranger". Defaults to "glm", which is a logit model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}). If "ranger", this function avoids propensity scores equal to 0 or 1 by bottom- and top-coding predicted values at .001 and .999.}

\item{outcome_algorithm}{Character name of the algorithm for the outcome model. One of "lm", "ridge", "gam", or "ranger". Defaults to "lm", which is an OLS model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}).}

\item{weight_name}{Character name of a sampling weight variable, if any, which captures the inverse probability of inclusion in the sample. The default assumes a simple random sample (all weights equal).}

\item{n_folds}{Only used if \code{method} = "cross_fit" and if \code{folds} is not provided. Integer scalar containing number of cross-validation folds. The function will assign observations to folds systematically: sort the data by the variable named \code{category_name}, then by the treatment variable, then at random. On this sorted dataset, folds are assigned systematically by repeated \code{1:n_folds}. To be used if the user does not provide \code{folds}. Defaults to 2.}

\item{folds_name}{Only used if \code{method} = "cross_fit". Character string indicating a column of \code{data} containing fold identifiers. This may be preferable to \code{n_folds} if the researcher has a reason to assign the folds in these data by some other process, perhaps due to particulars of how these data were generated. If null (the default), folds are assigned as stated in \code{n_folds}.}
}
\value{
A list with four elements.
\itemize{
\code{counterfactual_means} A tibble with a counterfactual mean estimate for each category
\code{counterfactual_means} A tibble with a counterfactual disparity estimate for each pair of categories
\code{treatment_model} Object containing the fitted treatment model
\code{outcome_model} Object containing the fitted outcome model
}
}
\description{
This is an internal function typically called from other functions rather than by the user. It creates cross-validation folds and repeatedly calls split_sample_estimator to conduct cross-fitting.
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_ridge.R
\name{fit_ridge}
\alias{fit_ridge}
\title{Ridge regression estimator}
\usage{
fit_ridge(data, model_formula, to_predict)
}
\arguments{
\item{data}{Data frame containing the observed data}

\item{model_formula}{A model formula object for the ridge regression to be fitted}

\item{to_predict}{Data frame containing observations for which predictions are to be made. If NULL, defaults to the same as \code{data}.}
}
\value{
A list containing a \code{glmnet} model object \code{fit} and a vector \code{fitted} of fitted values for observations in \code{to_predict}.
}
\description{
Not typically called by the user directly; called indirectly via other functions. Uses \code{glmnet} to fit a ridge regression with penalty chosen by cross-validation. Returns fitted values for the data in \code{to_predict}.
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.

Friedman J, Hastie T, Tibshirani R (2010). "Regularization Paths for Generalized Linear Models via Coordinate Descent." Journal of Statistical Software, 33(1), 1–22. {https://www.jstatsoft.org/htaccess.php?volume=33&type=i&issue=01}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gapclosing.R
\name{gapclosing}
\alias{gapclosing}
\title{Gap closing estimator}
\usage{
gapclosing(
  data,
  counterfactual_assignments,
  outcome_formula = NULL,
  treatment_formula = NULL,
  category_name,
  outcome_name = NULL,
  treatment_name = NULL,
  treatment_algorithm = "glm",
  outcome_algorithm = "lm",
  sample_split = "single_sample",
  se = FALSE,
  bootstrap_samples = 1000,
  bootstrap_method = "simple",
  parallel_cores = NULL,
  weight_name = NULL,
  n_folds = 2,
  folds_name = NULL
)
}
\arguments{
\item{data}{Data frame containing the observed data}

\item{counterfactual_assignments}{Numeric scalar or vector of length nrow(data), each element of which is on the [0,1] interval. If a scalar, the counterfactual probability by which all units are assigned to treatment condition 1. If a vector, each element i corresponds to the counterfactual probability by which each unit i is assigned to treatment condition 1.}

\item{outcome_formula}{Outcome formula , in the style \code{outcome ~ treatment*covariate}. Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph). If \code{outcome_algorithm} = "ranger", then the outcome model will be fit separately on the treatment and control groups. Otherwise, the user must specify all interactions in the formula.}

\item{treatment_formula}{Treatment formula, in the style \code{treatment ~ covariate}. Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph).}

\item{category_name}{Character name of the variable indicating the categories over which the gap is defined. Must be the name of a column in \code{data}.}

\item{outcome_name}{Character name of the outcome variable. Only required when there is no outcome_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.}

\item{treatment_name}{Character name of the treatment variable. Only required when there is no treatment_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.}

\item{treatment_algorithm}{Character name of the algorithm for the treatment model. One of "glm", "ridge", "gam", or "ranger". Defaults to "glm", which is a logit model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}). If "ranger", this function avoids propensity scores equal to 0 or 1 by bottom- and top-coding predicted values at .001 and .999.}

\item{outcome_algorithm}{Character name of the algorithm for the outcome model. One of "lm", "ridge", "gam", or "ranger". Defaults to "lm", which is an OLS model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}).}

\item{sample_split}{Character for the type of sample splitting to be conducted. One of "single_sample" or "cross_fit". Defaults to "single_sample", in which case \code{data} is used for both learning the nuisance functions and aggregating to an estimate. Option "cross_fit" uses cross-fitting to repeatedly use part of the sample to learn the nuisance function and another part to estimate the estimand, averaged over repetitions with these roles swapped.}

\item{se}{Logical indicating whether standard errors should be calculated. Default is FALSE. Standard errors assume a simple random sample by default; to stratify by (category x treatment), see the \code{bootstrap_method} argument. Because many datasets are not simple random samples, users should carefully consider whether a simple random sample bootstrap will accurately capture uncertainty.}

\item{bootstrap_samples}{Only used if \code{se = TRUE}. Number of bootstrap samples. Default is 1000.}

\item{bootstrap_method}{Only used if \code{se = TRUE}. A character string stating how to conduct bootstrap samples. If "simple", then samples are drawn with replacement from the full data. If "stratified", then the bootstrap is carried out within subpopulations defined by category and treatment. The latter may be useful if the sample contains only a small number of observations in these cells and the user wants to ensure that every (category x treatment) cell appears in every bootstrap sample. With "stratified", inference assumes that in repeated samples from the true population the proportion in each (category x treatment) cell would not change; this may or may not correspond to the true sampling process. Users should be cautious.}

\item{parallel_cores}{Integer number of cores for parallel processing of the bootstrap. Defaults to sequential processing.}

\item{weight_name}{Character name of a sampling weight variable, if any, which captures the inverse probability of inclusion in the sample. The default assumes a simple random sample (all weights equal).}

\item{n_folds}{Only used if \code{method} = "cross_fit" and if \code{folds} is not provided. Integer scalar containing number of cross-validation folds. The function will assign observations to folds systematically: sort the data by the variable named \code{category_name}, then by the treatment variable, then at random. On this sorted dataset, folds are assigned systematically by repeated \code{1:n_folds}. To be used if the user does not provide \code{folds}. Defaults to 2.}

\item{folds_name}{Only used if \code{method} = "cross_fit". Character string indicating a column of \code{data} containing fold identifiers. This may be preferable to \code{n_folds} if the researcher has a reason to assign the folds in these data by some other process, perhaps due to particulars of how these data were generated. If null (the default), folds are assigned as stated in \code{n_folds}.}
}
\value{
An object of S3 class \code{gapclosing}, which supports \code{summary()}, \code{print()}, and \code{plot()} functions. The returned object can be coerced to a data frame of estimates with \code{as.data.frame()}. \cr
The object returned by a call to \code{gapclosing} contains several elements.
\itemize{
\item \code{factual_means} A tibble containing the factual mean outcome in each category \cr
\item \code{factual_disparities} A tibble containing the disparities in factual mean outcomes across categories \cr
\item \code{counterfactual_means} A tibble containing the counterfactual mean outcome (post-intervention mean) in each category \cr
\item \code{counterfactual_disparities} A tibble containing the counterfactual disparities (gap-closing estimands) across categories \cr
\item \code{change_means} A tibble containing the additive and proportional change from factual to counterfactual values for mean outcomes \cr
\item \code{change_disparities} A tibble containing the additive and proportional change from factual to counterfactual values for disparities in mean outcomes (e.g. proportion of the factual gap which is closed by the intervention) \cr
\item \code{all_estimators} A list containing estimates by treatment modeling, outcome modeling, and doubly-robust estimation. If any of these are not applicable, estimates are NA. \cr
\item \code{primary_estimator_name} The name of the primary estimator (treatment_modeling, outcome_modeling, or doubly_robust). The estimates reported in the first 6 slots of the returned object come from this estimator. \cr
\item \code{treatment_model} The fitted treatment model (or models on each fold in the case of cross-fitting). Note that this model object is a point estimate with standard errors derived from the algorithm used to fit it; any standard errors within \code{treatment_model} do not come from bootstrapping by the package. \cr
\item \code{outcome_model} The fitted outcome model (or models on each fold in the case of cross-fitting). Note that this model object is a point estimate with standard errors derived from the algorithm used to fit it; any standard errors within \code{treatment_model} do not come from bootstrapping by the package. \cr
\item \code{call} The call that produced this \code{gapclosing} object \cr
\item \code{arguments} A list of all arguments from the call to \code{gapclosing}
}
}
\description{
A function to estimate gap-closing estimands: means and disparities across categories of units that would persist under some counterfactual assignment of a treatment. To use this function, the user provides a data frame \code{data}, a rule \code{counterfactual_assignments} for counterfactually assigning treatment, a treatment and/or an outcome model for learning statistically about the counterfactuals, and the \code{category_name} of the variable in \code{data} over which categories are defined. The returned object summarizes factual and counterfactual means and disparities. Supported estimation algorithms include generalized linear models, ridge regression, generalized additive models, and random forests. Standard errors are supported by bootstrapping.
}
\examples{
# Simulate example data
simulated_data <- generate_simulated_data(n = 100)

# Fit by outcome modeling
# You can add standard errors with se = TRUE
estimate <- gapclosing(
  data = simulated_data,
  outcome_formula = outcome ~ treatment * category + confounder,
  treatment_name = "treatment",
  category_name = "category",
  counterfactual_assignments = 1
)
summary(estimate)

# Fit by treatment modeling
# You can add standard errors with se = TRUE
estimate <- gapclosing(
  data = simulated_data,
  treatment_formula = treatment ~ category + confounder,
  outcome_name = "outcome",
  category_name = "category",
  counterfactual_assignments = 1
)
summary(estimate)

# Fit by doubly-robust estimation
# You can add standard errors with se = TRUE
estimate <- gapclosing(
  data = simulated_data,
  outcome_formula = outcome ~ treatment * category + confounder,
  treatment_formula = treatment ~ category + confounder,
  category_name = "category",
  counterfactual_assignments = 1
)
summary(estimate)

# Fit by doubly-robust cross-fitting estimation with random forests
# You can add standard errors with se = TRUE
estimate <- gapclosing(
  data = simulated_data,
  outcome_formula = outcome ~ category + confounder,
  treatment_formula = treatment ~ category + confounder,
  category_name = "category",
  counterfactual_assignments = 1,
  outcome_algorithm = "ranger",
  treatment_algorithm = "ranger",
  sample_split = "cross_fit"
)
summary(estimate)
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.

Friedman J, Hastie T, Tibshirani R (2010). "Regularization Paths for Generalized Linear Models via Coordinate Descent." Journal of Statistical Software, 33(1), 1–22. {https://www.jstatsoft.org/htaccess.php?volume=33&type=i&issue=01}.

Wood S (2017). Generalized Additive Models: An Introduction with R, 2 edition. Chapman and Hall/CRC.

Wright MN, Ziegler A (2017). "ranger: A Fast Implementation of Random Forests for High Dimensional Data in C++ and R." Journal of Statistical Software, 77(1), 1–17. doi: 10.18637/jss.v077.i01.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_simulated_data.R
\name{generate_simulated_data}
\alias{generate_simulated_data}
\title{Generate simulated data}
\usage{
generate_simulated_data(n = 1000)
}
\arguments{
\item{n}{Number of observations to be generated}
}
\value{
A data frame with \code{n} rows and 4 columns containing simulated data containing \code{category} over which disparities are defined, a \code{confounder} that affects treatment assignment, a binary \code{treatment}, and a continuous \code{outcome}.
}
\description{
Generates simulated data to illustrate the gapclosing function
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/point_estimator.R
\name{point_estimator}
\alias{point_estimator}
\title{Point estimator for gap-closing estimands}
\usage{
point_estimator(
  data_learn,
  data_estimate,
  counterfactual_assignments,
  outcome_formula,
  treatment_formula,
  category_name,
  outcome_name,
  treatment_name,
  treatment_algorithm = "glm",
  outcome_algorithm = "lm",
  weight_name = NULL
)
}
\arguments{
\item{data_learn}{Data frame in which treatment and outcome models will be learned}

\item{data_estimate}{Data frame in which the learned models will be converted to an estimate of the gap-closing estimand}

\item{counterfactual_assignments}{Numeric scalar or vector of length nrow(data), each element of which is on the [0,1] interval. If a scalar, the counterfactual probability by which all units are assigned to treatment condition 1. If a vector, each element i corresponds to the counterfactual probability by which each unit i is assigned to treatment condition 1.}

\item{outcome_formula}{Model formula the outcome. Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph). If \code{outcome_algorithm} = "ranger", then the outcome model will be fit separately on the treatment and control groups. Otherwise, the user must specify all interactions in the formula.}

\item{treatment_formula}{Treatment formula, in the style formula(treatment ~ covariates). Covariates should include those needed for causal identification of the treatment effect (e.g. as defended in your Directed Acyclic Graph).}

\item{category_name}{Character name of the variable indicating the categories over which the gap is defined. Must be the name of a column in \code{data}.}

\item{outcome_name}{Character name of the outcome variable. Only required when there is no outcome_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.}

\item{treatment_name}{Character name of the treatment variable. Only required when there is no treatment_formula; otherwise extracted automatically. Must be a name of a column in \code{data}.}

\item{treatment_algorithm}{Character name of the algorithm for the treatment model. One of "glm", "ridge", "gam", or "ranger". Defaults to "glm", which is a logit model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}). If "ranger", this function avoids propensity scores equal to 0 or 1 by bottom- and top-coding predicted values at .001 and .999.}

\item{outcome_algorithm}{Character name of the algorithm for the outcome model. One of "lm", "ridge", "gam", or "ranger". Defaults to "lm", which is an OLS model. Option "ridge" is ridge regression. Option "gam" is a generalized additive model fit (see package \code{mgcv}). Option "ranger" is a random forest (see package \code{ranger}).}

\item{weight_name}{Character name of a sampling weight variable, if any, which captures the inverse probability of inclusion in the sample. The default assumes a simple random sample (all weights equal).}
}
\value{
@return A list with four elements.
\itemize{
\code{counterfactual_means} A tibble with a counterfactual mean estimate for each category
\code{counterfactual_means} A tibble with a counterfactual disparity estimate for each pair of categories
\code{treatment_model} Object containing the fitted treatment model
\code{outcome_model} Object containing the fitted outcome model
}
}
\description{
This is an internal function typically called from other functions rather than by the user. It uses a learning sample to learn the nuisance functions (treatment and outcome model) and then an auxiliary estimation sample to use those functions in estimation of the gap-closing estimand. For single-sample estimation, both the learning and estimation samples are the same. For cross-fitting, this function is called repeatedly with the roles of each dataset swapped.
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disparityplot.R
\name{disparityplot}
\alias{disparityplot}
\title{Plot a disparity}
\usage{
disparityplot(
  x,
  category_A,
  category_B,
  custom_ylab = "Mean Outcome",
  custom_xlab = "Category"
)
}
\arguments{
\item{x}{An object of class \code{gapclosing}, which results from a call to the function \code{gapclosing}}

\item{category_A}{The first category to be plotted. A value of the category_name variable in x.}

\item{category_B}{The second category to be plotted. Must be a value of \code{x$category}}

\item{custom_ylab}{Custom y-axis label. Defaults to "Mean Outcome"}

\item{custom_xlab}{Custom x-axis label. Defaults to "Category"}
}
\value{
A \code{ggplot2} object
}
\description{
Plots the factual and counterfactual mean outcomes in two categories. The returned object is a ggplot2 object which can be further customized using the syntax of ggplot2.
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/df_to_gapclosing_list.R
\name{df_to_gapclosing_list}
\alias{df_to_gapclosing_list}
\title{Convert Back to Canonical List Output}
\usage{
df_to_gapclosing_list(x)
}
\arguments{
\item{x}{A data frame produced by \code{as.data.frame(x)} applied to an object \code{x} of class \code{gapclosing}.}
}
\value{
A list containing a subset of the elements in a \code{gapclosing} object.
}
\description{
If the user has used \code{as.data.frame(x)} to convert a \code{gapclosing} object to a data frame of estimates, this function will invert back to the original list format. This function does not fully reinstate the original gapclosing object because some elements are lost when \code{as.data.frame()} is called. This function is most useful as a check on \code{as.data.frame()} and as a helper in settings like bootstrapping where a data frame is easier to work with but we want to return to the original format before returning an object to the user.
}
\examples{
# Simulate example data
simulated_data <- generate_simulated_data(n = 100)

# Fit by outcome modeling
estimate <- gapclosing(
  data = simulated_data,
  outcome_formula = formula(outcome ~ treatment * category + confounder),
  treatment_name = "treatment",
  category_name = "category",
  counterfactual_assignments = 1
)
summary(estimate)

# Convert to a data frame
estimate.df <- as.data.frame(estimate)
# Convert back to a list
estimate.df <- df_to_gapclosing_list(estimate.df)
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.gapclosing.R
\name{summary.gapclosing}
\alias{summary.gapclosing}
\title{Summary function for gapclosing objects}
\usage{
\method{summary}{gapclosing}(object, ...)
}
\arguments{
\item{object}{An object of class \code{gapclosing}, which results from a call to the function \code{gapclosing}}

\item{...}{Other arguments to \code{summary} commands}
}
\value{
Prints a summary of the estimates.
}
\description{
Summarizes the S3 class object returned by the gapclosing function
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.gapclosing.R
\name{as.data.frame.gapclosing}
\alias{as.data.frame.gapclosing}
\title{Coerce to a Data Frame}
\usage{
\method{as.data.frame}{gapclosing}(x, ...)
}
\arguments{
\item{x}{Object of class \code{gapclosing}, produced by a call to \code{gapclosing()}.}

\item{...}{Additional arguments to be passed to or from methods.}
}
\value{
A data frame containing estimates.
}
\description{
This function converts a \code{gapclosing} object into a data frame. The gapclosing class contains results within a named list, thus simplifying things for manual user interaction with the results. In some programming settings (e.g. a bootstrap), it is easier to work with a rectangular data frame of results. This function produces that data frame.
}
\examples{
# Simulate example data
simulated_data <- generate_simulated_data(n = 100)

# Fit by outcome modeling
estimate <- gapclosing(
  data = simulated_data,
  outcome_formula = formula(outcome ~ treatment * category + confounder),
  treatment_name = "treatment",
  category_name = "category",
  counterfactual_assignments = 1
)
summary(estimate)

# Convert to a data frame
estimate.df <- as.data.frame(estimate)
}
\references{
Lundberg I (2021). "The gap-closing estimand: A causal approach to study interventions that close disparities across social categories." Sociological Methods and Research. Available at {https://osf.io/gx4y3/}.
}
